//
//  KColorSyntaxServer.h
//  KEdit
//
//  Created by Karino, Masatsugu on Mon May 26 2003.
//  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "MyDocument.h"

// Setting file names. Extensions must be 'txt' currently.
#define KModeSettingFileName_HTML @"html_settings"
#define KKeywordsFileName_HTML @"html_keywords"
#define KModeSettingsFileName_PHP @"php_settings"
#define KKeywordsFileName_PHP @"php_keywords"
#define KModeSettingsFileName_Perl @"perl_settings"
#define KKeywordsFileName_Perl @"perl_keywords"
#define KModeSettingsFileName_CSS @"css_settings"
#define KKeywordsFileName_CSS @"css_keywords"
#define KModeSettingsFileName_IndexedText @"i-text_settings"
#define KKeywordsFileName_IndexedText @"i-text_keywords"
#define KModeSettingsFileName_Ruby @"ruby_settings"
#define KKeywordsFileName_Ruby @"ruby_keywords"

#define KModeSettingsFileName_KIndex @"k-index_settings"

@class KModeSettings;
@class KKeywords;

// An object for treating color syntax mode.
// The instance of this will be attached to MyDocument.
@interface KColorSyntaxServer : NSObject {
	KModeSettings* mSettings;
	KKeywords* mKeywords;
	NSTextStorage* mTextStorage;
	
	KTextType mTextType;
	unsigned mPrevCommentCount;
}

- (id)initWithTextStorage:(NSTextStorage*)storage ofType:(KTextType)type;
+ (id)colorSyntaxServerWithTextStorage:(NSTextStorage*)storage ofType:(KTextType)type;

// color validation for the document.
- (void)adjustColorSyntaxWithRange:(NSRange)validatedRange andFont:(NSFont*)font;

// accessor methods.
- (KTextType)textType;
- (void)setTextType:(KTextType)type;
- (NSTextStorage*)textStorage;
- (void)setTextStorage:(NSTextStorage*)storage;

- (KModeSettings*)settings;

// INTERNAL USE ONLY.

// mode-specific internal methods.
- (void)validateColorSyntaxPlain:(NSRange)validatedRange;
- (void)validateColorSyntaxHTML:(NSRange)validatedRange;
- (void)validateColorSyntaxPHP:(NSRange)validatedRange;
- (void)validateColorSyntaxPerl:(NSRange)validatedRange;
- (void)validateColorSyntaxRuby:(NSRange)validatedRange;
- (void)validateColorSyntaxCSS:(NSRange)validatedRange;
- (void)validateColorSyntaxIndexed:(NSRange)validatedRange;
- (void)validateColorSyntaxKMarkup:(NSRange)validatedRange;

// internal use.
- (void)resetSettings;

- (void)setQuoteAndCommentColorsWithRange:(NSRange)validatedRange comment:(NSString*)comment;
- (void)setMultiLineCommentColorWithHead:(NSString*)head tail:(NSString*)tail;
- (void)setMultiLineCommentColorWithHead:(NSString*)head tail:(NSString*)tail color:(NSColor*)color ignoreQuotes:(BOOL)ignoreQuotes;
- (void)setLineColorWithRange:(NSRange)validatedRange startsWith:(NSString*)string color:(NSColor*)color;

- (void)setForegroundColorWith:(NSColor*)color range:(NSRange)range;

	//within whole string.
	// if no change, return NO; otherwise YES. mPrevCommentCount will be adjusted.
- (BOOL)checkMultiLineCommentCountWithHead:(NSString*)head tail:(NSString*)tail;
- (BOOL)checkMultiLineCommentCountWithHead:(NSString*)head tail:(NSString*)tail ignoreQuotes:(BOOL)ignoreQuotes;

@end





