#import "KKeyAssignServer.h"
#import "KApplicationDel.h"
#import "KPrefsServer.h"
#import "KTextView.h"

@implementation KKeyAssignServer

- (id)init
{
	[super init];
	
	mPrevCharacter = [[NSString alloc] initWithString:@""];
	mModifierFlags = 0;
	
	return self;
}

- (void)dealloc
{
	[mPrevCharacter release];
}

- (BOOL)doneAssignedFunction:(NSEvent*)event sender:(id)sender
{
	/*
	NSAlphaShiftKeyMask - CapsLock
	NSShiftKeyMask - shift
	NSControlKeyMask - control
	NSAlternateKeyMask - option
	NSCommandKeyMask - command
	
	NSUpArrowFunctionKey - up arrow
	NSDownArrowFunctionKey - down arrow
	NSLeftArrowFunctionKey - left arrow
	NSRightArrowFunctionKey - right arrow
	
	NSDeleteFunctionKey - forward delete
	NSPageUpFunctionKey - page up
	NSPageDownFunctionkey - page down
	*/
	
	KPrefsServer* prefs = [[NSApp delegate] prefsServer];
	
	NSString* currentChar = [[NSString alloc] initWithString:[[event charactersIgnoringModifiers] uppercaseString]];
	unsigned int currentModifiers = [event modifierFlags];
	KKeyBind keyBind = [prefs intForKey:KKeyBindType];
	BOOL doneFunction = NO;
	
	// KEYBINDs
	if(keyBind == KKEditOriginal){
		doneFunction = [self doneKEditOriginalFunction:currentChar modifiers:currentModifiers sender:sender];
	}else if(keyBind == KEmacsLike){
		doneFunction = [self doneEmacsLikeFunction:currentChar modifiers:currentModifiers sender:sender];
	}else if(keyBind == KViLike){
		doneFunction = [self doneViLikeFunction:currentChar modifiers:currentModifiers sender:sender];
	}else{
		NSLog(@"KKeyAssignServer - doneAssignedFunction: / an error has occured.");
	}
	
	[mPrevCharacter release];
	mPrevCharacter = currentChar;
	mModifierFlags = currentModifiers;
	
	// AUTOINDENT
	if([currentChar isEqualToString:KRETURN_char] && [sender isAutoIndented]){
		[sender autoIndent:self];
		doneFunction = YES;
	}
	
	return doneFunction;
}

- (BOOL)doneKEditOriginalFunction:(NSString*)character modifiers:(unsigned int)mask sender:(id)sender
{
	if((mask & NSControlKeyMask) && !(mask & NSAlternateKeyMask)){// CONTROL(+), OPTION(-)
		if(!(mask & NSShiftKeyMask)){// SHIFT(-)
			if([mPrevCharacter isEqualToString:@"Q"] && ((mModifierFlags & NSControlKeyMask) && !(mModifierFlags & NSAlternateKeyMask))){
				// prevKey is ctrl-'Q'
				if([character isEqualToString:@"R"]){ [sender moveToBeginningOfDocument:self]; return YES; }
				if([character isEqualToString:@"C"]){ [sender moveToEndOfDocument:self]; return YES; }
				return YES;
			}
			if([character isEqualToString:@"Q"]){ return YES; }
			
			if([character isEqualToString:@"A"]){ [sender moveToBeginningOfParagraph:self]; return YES; }
			if([character isEqualToString:@"S"]){ [sender moveBackward:self]; return YES; }
			if([character isEqualToString:@"D"]){ [sender moveForward:self]; return YES; }
			if([character isEqualToString:@"F"]){ [sender moveToEndOfParagraph:self]; return YES; }
			
			if([character isEqualToString:@"E"]){ [sender moveUp:self]; return YES; }
			if([character isEqualToString:@"X"]){ [sender moveDown:self]; return YES; }
			
			if([character isEqualToString:@"R"]){ [sender pageUp:self]; return YES; }
			if([character isEqualToString:@"C"]){ [sender pageDown:self]; return YES; }
			
			if([character isEqualToString:@"H"]){ [sender deleteBackward:self]; return YES; }
			if([character isEqualToString:@"G"]){ [sender deleteForward:self]; return YES; }
			if([character isEqualToString:@"Y"]){ [sender deleteToEndOfParagraph:self];
												  [sender deleteToBeginningOfParagraph:self]; return YES; }
			
			if([character isEqualToString:@"I"]){ [sender insertTab:self]; return YES; }
			if([character isEqualToString:@"M"]){ [sender insertNewline:self]; return YES; }
		
			if([character isEqualToString:@"U"]){ [sender uppercaseWord:self]; return YES; }
			if([character isEqualToString:@"L"]){ [sender lowercaseWord:self]; return YES; }
			
			if([character isEqualToString:@"P"]){ [sender doSample:self]; return YES; }
			
		}else{ // SHIFT(+)
			if([character isEqualToString:@"A"]){ [sender moveToBeginningOfParagraphAndModifySelection:self]; return YES; }//KTextView only
			if([character isEqualToString:@"S"]){ [sender moveBackwardAndModifySelection_kedit:self]; return YES; }//KTextView only
			if([character isEqualToString:@"D"]){ [sender moveForwardAndModifySelection_kedit:self]; return YES; }//KTextView only
			if([character isEqualToString:@"F"]){ [sender moveToEndOfParagraphAndModifySelection:self]; return YES; }//KTextView only
		}
	}
	return NO;
}

- (BOOL)doneEmacsLikeFunction:(NSString*)character modifiers:(unsigned int)mask sender:(id)sender
{
	if([character isEqualToString:@"\x1b"]){// ESCAPE KEY...
		return YES;
	}else if([mPrevCharacter isEqualToString:KESC_char]){// prev key is ESCAPE.
		if([character isEqualToString:@"D"]){ [sender deleteWordBackward:self]; return YES; }
		if([character isEqualToString:@"W"]){ [sender copy:self]; return YES; }
		if([character isEqualToString:@"V"]){ [sender pageUp:self]; return YES; }
		
		if([character isEqualToString:@"U"]){ [sender uppercaseWord:self]; return YES; }
		if([character isEqualToString:@"L"]){ [sender lowercaseWord:self]; return YES; }
		
		if((mask & NSControlKeyMask) && !(mask & NSAlternateKeyMask)){// CONTROL(+), OPTION(-)
			if([character isEqualToString:@"K"]){ [sender deleteToEndOfLine:self]; return YES; }
		}
		return NO;
	}
	
	
	if((mask & NSControlKeyMask) && !(mask & NSAlternateKeyMask)){// CONTROL(+), OPTION(-)
		if(!(mask & NSShiftKeyMask)){// SHIFT(-)
			if([mPrevCharacter isEqualToString:@"X"] && ((mModifierFlags & NSControlKeyMask) && !(mModifierFlags & NSAlternateKeyMask))){
				// prevKey is ctrl-'X'
				if([character isEqualToString:@"S"]){ [sender saveDocument:self]; return YES; }
				return YES;
			}
			if([character isEqualToString:@"X"]){ return YES; }
			
			if([character isEqualToString:@"P"]){ [sender moveUp:self]; return YES; }
			if([character isEqualToString:@"N"]){ [sender moveDown:self]; return YES; }
			if([character isEqualToString:@"B"]){ [sender moveBackward:self]; return YES; }
			if([character isEqualToString:@"F"]){ [sender moveForward:self]; return YES; }
			
			if([character isEqualToString:@"A"]){ [sender moveToBeginningOfLine:self]; return YES; }
			if([character isEqualToString:@"E"]){ [sender moveToEndOfLine:self]; return YES; }
			
			if([character isEqualToString:@"R"]){ [sender moveToBeginningOfDocument:self]; return YES; }
			if([character isEqualToString:@"C"]){ [sender moveToEndOfDocument:self]; return YES; }
			
			if([character isEqualToString:@"V"]){ [sender pageDown:self]; return YES; }
			
			if([character isEqualToString:@"H"]){ [sender deleteBackward:self]; return YES; }
			if([character isEqualToString:@"D"]){ [sender deleteForward:self]; return YES; }
			
			if([character isEqualToString:@"K"]){ [sender deleteToBeginningOfLine:self]; return YES; }
			
			if([character isEqualToString:@"I"]){ [sender insertTab:self]; return YES; }
			if([character isEqualToString:@"M"]){ [sender insertNewline:self]; return YES; }
			
			if([character isEqualToString:@"W"]){ [sender cut:self]; return YES; }
			if([character isEqualToString:@"Y"]){ [sender paste:self]; return YES; }
			
		}else{ // SHIFT(+)
			
		}
	}
	return NO;
}

- (BOOL)doneViLikeFunction:(NSString*)character modifiers:(unsigned int)mask sender:(id)sender
{
	if((mask & NSControlKeyMask) && !(mask & NSAlternateKeyMask)){ // CONTROL(+), OPTION(-)
		if([character isEqualToString:@"K"]){ [sender moveUp:self]; return YES; }
		if([character isEqualToString:@"J"]){ [sender moveDown:self]; return YES; }
		if([character isEqualToString:@"H"]){ [sender moveBackward:self]; return YES; }
		if([character isEqualToString:@"L"]){ [sender moveForward:self]; return YES; }
		
		if([character isEqualToString:@"X"]){ [sender deleteBackward:self]; return YES; }
		
		//if([character isEqualToString:@"U"]){ [sender undo:self]; return YES; }
	}
	return NO;
}



@end

