/* KMenuUtils */

#import <Cocoa/Cocoa.h>

#define KIEBookmarkFileName @"Library/Preferences/Explorer/Favorites.html"
#define KSafariBookmarkFileName @"Library/Safari/Bookmarks.plist"

// zero-terminated float list.
static const float KFontSizeMenuItemArray[] = { 6, 8, 9, 10, 12, 14, 18, 24, 36, 48, 0 };

@interface KMenuUtils : NSObject
{
}
// Search a menuitem which title is localizedString.
// If no such item, return nil.
+ (NSMenuItem*)searchMenuItemOfMenu:(NSMenu*)targetMenu localizedString:(NSString*)localizedString;

// Search a menuitem which tag is 'tag'.
// If no such item, return nil.
+ (NSMenuItem*)searchMenuItemWithTag:(int)tag targetMenu:(NSMenu*)targetMenu;

// Get a menuitem which submenu is targetMenu.
// If no such item, return nil.
+ (NSMenuItem*)getMenuItemWithSubmenu:(NSMenu*)targetMenu;

// Set states of all menuitems of targetMenu and its submenus to NSOffState
+ (void)turnOffAllMenuItemState:(NSMenu*)targetMenu;

// Attach a font list menu to targetMenuItem as submenu.
// You can set selecter to 'selector'.
// This method returnes font names array.
// The tag of menuitem added is index of the font name at the array.
+ (NSArray*)attachFontMenuToMenuItem:(NSMenuItem*)targetMenuItem selector:(SEL)selector;
+ (NSArray*)attachFontMenuToMenu:(NSMenu*)targetMenu selector:(SEL)selector;
// The methods below are construct flatted(family - face) font menu.
+ (NSArray*)attachFlattedFontMenuToMenuItem:(NSMenuItem*)targetMenuItem selector:(SEL)selector;
+ (NSArray*)attachFlattedFontMenuToMenu:(NSMenu*)targetMenu selector:(SEL)selector;

// Attach a font size menu to targetMenuItem as submenu.
+ (NSArray*)attachSizeMenuToMenuItem:(NSMenuItem*)targetMenuItem selector:(SEL)selector;
+ (NSArray*)attachSizeMenuToMenu:(NSMenu*)targetMenu selector:(SEL)selector;

// Attach a tag menu to targetMenuItem as submenu.
+ (void)attachTagMenuToMenuItem:(NSMenuItem*)targetMenuItem selector:(SEL)selector;
+ (void)attachTagMenuToMenu:(NSMenu*)targetMenu selector:(SEL)selector;

// Attach a URL menu to targetMenuItem as submenu.
+ (void)attachURLMenuToMenuItem:(NSMenuItem*)targetMenuItem selector:(SEL)selector;
+ (void)attachURLMenuToMenu:(NSMenu*)targetMenu selector:(SEL)selector;

// Attach a Tools(Scripts) menu to targetMenuItem as submenu.
+ (void)attachScriptsMenuToMenuItem:(NSMenuItem*)targetMenuItem selector:(SEL)selector;
+ (void)attachScriptsMenuToMenu:(NSMenu*)targetMenu selector:(SEL)selector;


// internal use only.
+ (void)constructTagMenu:(NSMenu*)targetMenu selector:(SEL)selector tagArray:(NSArray*)tagArray;
+ (void)constructScriptMenu:(NSMenu*)targetMenu dir:(NSString*)pathName selector:(SEL)selector;
+ (void)constructURLMenuWithIEBookmarks:(NSMenu*)targetMenu selector:(SEL)selector;
+ (void)constructURLMenuWithSafariBookmarks:(NSMenu*)targetMenu selector:(SEL)selector;

+ (void)constructURLMenuWithSafariBookmarksWithArray:(NSMenu*)targetMenu selector:(SEL)selector array:(NSArray*)array;
@end


@interface KMenuItem : NSMenuItem
{
	id mAttachedObj;
}
- (id)initWithTitle:(NSString*)itemName action:(SEL)anAction keyEquivalent:(NSString*)charCode;
- (void)dealloc;

- (id)attachedObj;
- (void)setAttachedObj:(id)obj;
@end










