#import "KMenuUtils.h"
#import "KStringUtils.h"
#import "KPrefsServer.h"

@implementation KMenuUtils

+ (NSMenuItem*)searchMenuItemOfMenu:(NSMenu*)targetMenu localizedString:(NSString*)localizedString
{
	NSMenuItem* menuItem;
	int i;
	for(i = 0; i < [targetMenu numberOfItems]; i++){
		menuItem = [targetMenu itemAtIndex:i];
		if([[menuItem title] isEqualToString:NSLocalizedString(localizedString, @"")]){
			return menuItem;
		}else if([menuItem hasSubmenu]){
			menuItem = [self searchMenuItemOfMenu:[menuItem submenu] localizedString:localizedString];
			if(menuItem){
				return menuItem;
			}
		}
	}
	return nil;
}

+ (void)turnOffAllMenuItemState:(NSMenu*)targetMenu
{
	NSMenuItem* menuItem;
	int i;
	for(i = 0; i < [targetMenu numberOfItems]; i++){
		menuItem = [targetMenu itemAtIndex:i];
		[menuItem setState:NSOffState];
		if([menuItem hasSubmenu]){
			[self turnOffAllMenuItemState:[menuItem submenu]];
		}
	}
}

+ (NSMenuItem*)searchMenuItemWithTag:(int)tag targetMenu:(NSMenu*)targetMenu
{
	NSMenuItem* menuItem, *returnedItem;
	
	int i;
	for(i = 0; i < [targetMenu numberOfItems]; i++){
		menuItem = [targetMenu itemAtIndex:i];
		if([menuItem tag] == tag){ return menuItem; }
		if([menuItem hasSubmenu]){
			returnedItem = [self searchMenuItemWithTag:tag targetMenu:[menuItem submenu]];
			if(returnedItem){ return returnedItem; }
		}
	}
	return nil;
}

+ (NSMenuItem*)getMenuItemWithSubmenu:(NSMenu*)targetMenu
{
	NSMenu* upperMenu = [targetMenu supermenu];
	if(!upperMenu){ return nil; }
	
	NSMenuItem* upperMenuItem;
	int i;
	for(i = 0; i < [upperMenu numberOfItems]; i++){
		upperMenuItem = [upperMenu itemAtIndex:i];
		if([upperMenuItem hasSubmenu]){
			if([upperMenuItem submenu] == targetMenu){
				return upperMenuItem;
			}
		}
	}
	return nil;
}


+ (NSMenu*)makeFontMenuHasSelector:(SEL)selector fontArray:(NSArray**)fontArray
{
	/*
	NSMenu* mainMenu = [NSApp mainMenu];
	NSMenu* fontMenu = [[NSMenu alloc] initWithTitle:@"font_menu"];
	NSFontManager* fontManager = [NSFontManager sharedFontManager];
	NSMutableArray* murableFontArray = [[NSMutableArray alloc]initWithCapacity:200];
	
	NSArray* systemFontArray = [fontManager availableFonts];
	NSArray* nonCharacterFontArray = [fontManager availableFontNamesWithTraits:(NSNonstandardCharacterSetFontMask)];
	
	int i, j;
	int tag = 0;
	NSString* fontName;
	for(i = 0; i < [systemFontArray count]; i++){
		fontName = [systemFontArray objectAtIndex:i];
		for(j = 0; j < [nonCharacterFontArray count]; j++){
			if(![fontName isEqualToString:[nonCharacterFontArray objectAtIndex:j]]
				&& ![[fontName substringWithRange:NSMakeRange(0, 1)] isEqualToString:@"."]){
				[mutableFontArray addObject:fontName];
			}
		}
	}
	
	NSArray* resArray = [[NSArray alloc] initWithArray:mutableFontArray];
	fontArray = &resArray;
	[murableFontArray release];
	
	return fontMenu;
	*/
	return nil;
}

+ (NSArray*)attachFontMenuToMenu:(NSMenu*)targetMenu selector:(SEL)selector
{
	NSFontManager* fontManager = [NSFontManager sharedFontManager];
	NSMutableArray* mutableFontArray = [[NSMutableArray alloc] initWithCapacity:200];
	
	int i, j;
	int tag = 0;
	NSMenu* memberMenu;
	NSMenuItem* fontMenuItem;
	KMenuItem* memberMenuItem;
	NSArray* fontArray, *memberArray;
	NSString* fontName;
	//NSDictionary* dict;
	
	fontArray = [fontManager availableFontFamilies];
	fontArray = [fontArray sortedArrayUsingSelector:@selector(localizedCaseInsensitiveCompare:)];
	for(i = 0; i < [fontArray count]; i++){
		fontName = [fontArray objectAtIndex:i];
		if([[fontName substringWithRange:NSMakeRange(0, 1)] isEqualToString:@"."]){ continue; }
		
		fontMenuItem = [targetMenu addItemWithTitle:[fontManager localizedNameForFamily:fontName face:nil] action:nil keyEquivalent:@""];
		memberMenu = [[NSMenu alloc] initWithTitle:@"member_submenu"];
		memberArray = [fontManager availableMembersOfFontFamily:fontName];
		for(j = 0; j < [memberArray count]; j++){
			memberMenuItem = [[KMenuItem alloc] initWithTitle:[[memberArray objectAtIndex:j] objectAtIndex:1] action:selector keyEquivalent:@""];
			//dict = [NSDictionary dictionaryWithObjectsAndKeys:[[memberArray objectAtIndex:j] objectAtIndex:0], @"font_name", nil];
			//[memberMenuItem setAttachedObj:dict];
			[memberMenu addItem:memberMenuItem];
			//memberMenuItem = [memberMenu addItemWithTitle:[[memberArray objectAtIndex:j] objectAtIndex:1] action:selector keyEquivalent:@""];
			
			[memberMenuItem setTag:tag];
			[mutableFontArray addObject:[[memberArray objectAtIndex:j] objectAtIndex:0]];
			tag++;
		}
		[fontMenuItem setSubmenu:memberMenu];
	}
	
	NSArray* retArray = [[NSArray alloc] initWithArray:mutableFontArray];
	[mutableFontArray release];
	
	return retArray;
}


+ (NSArray*)attachFontMenuToMenuItem:(NSMenuItem*)targetMenuItem selector:(SEL)selector
{
	NSMenu* fontMenu = [[NSMenu alloc] initWithTitle:@"font_submenu"];
	[targetMenuItem setSubmenu:fontMenu];
	
	return [self attachFontMenuToMenu:fontMenu selector:selector];
}

+ (NSArray*)attachFlattedFontMenuToMenu:(NSMenu*)targetMenu selector:(SEL)selector
{
	NSFontManager* fontManager = [NSFontManager sharedFontManager];
	NSMutableArray* mutableFontArray = [[NSMutableArray alloc] initWithCapacity:200];
	
	int i;
	int tag = 0;
	NSMenuItem* fontMenuItem;
	NSArray* fontArray;
	NSString* fontName;
	
	fontArray = [fontManager availableFonts];
	fontArray = [fontArray sortedArrayUsingSelector:@selector(localizedCaseInsensitiveCompare:)];
	for(i = 0; i < [fontArray count]; i++){
		fontName = [fontArray objectAtIndex:i];
		if([[fontName substringWithRange:NSMakeRange(0, 1)] isEqualToString:@"."]){ continue; }
		
		fontMenuItem = [targetMenu addItemWithTitle:[fontManager localizedNameForFamily:fontName face:nil] action:selector keyEquivalent:@""];
		[fontMenuItem setTag:tag];
		[mutableFontArray addObject:fontName];
		tag++;
	}
	
	NSArray* retArray = [[NSArray alloc] initWithArray:mutableFontArray];
	[mutableFontArray release];
	
	return retArray;

}

+ (NSArray*)attachFlattedFontMenuToMenuItem:(NSMenuItem*)targetMenuItem selector:(SEL)selector
{
	NSMenu* fontMenu = [[NSMenu alloc] initWithTitle:@"font_submenu"];
	[targetMenuItem setSubmenu:fontMenu];
	
	return [self attachFlattedFontMenuToMenu:fontMenu selector:selector];
}

+ (NSArray*)attachSizeMenuToMenu:(NSMenu*)targetMenu selector:(SEL)selector
{
	NSMutableArray* sizeArray = [[NSMutableArray alloc] initWithCapacity:20];
	int i;
	float size;
	NSMenuItem* menuItem;
	
	while(size = KFontSizeMenuItemArray[i]){
		[sizeArray addObject:[NSNumber numberWithFloat:size]];
		menuItem = [targetMenu addItemWithTitle:[NSString stringWithFormat:@"%d", (int)size] action:selector keyEquivalent:@""];
		[menuItem setTag:i];
		i++;
	}
	
	return sizeArray;
}

+ (NSArray*)attachSizeMenuToMenuItem:(NSMenuItem*)targetMenuItem selector:(SEL)selector
{
	NSMenu* sizeMenu = [[NSMenu alloc] initWithTitle:@"size_submenu"];
	[targetMenuItem setSubmenu:sizeMenu];
	
	return [self attachSizeMenuToMenu:sizeMenu selector:selector];
}

+ (void)attachTagMenuToMenu:(NSMenu*)targetMenu selector:(SEL)selector
{
	NSFileManager* manager = [NSFileManager defaultManager];
	NSString* dirName = [NSHomeDirectory() stringByAppendingPathComponent:KPreferenceDirectory];
	NSString* tagFileName = [dirName stringByAppendingPathComponent:KTagFileName];
	
	if(![manager fileExistsAtPath:tagFileName]){
		NSBundle* bundle = [NSBundle mainBundle];
		tagFileName = [bundle pathForResource:@"html_tags" ofType:@"txt"];
		if(!tagFileName){ NSLog(@"KMenuUtils : tagFileName == nil"); return; }
	}
	
	NSString* tagFileContents = [NSString stringWithContentsOfFile:tagFileName];
	if(!tagFileContents){ NSLog(@"KMenuUtils : tagFileContents == nil"); return; }
	
	NSArray* commandArray = [KStringUtils string:tagFileContents componentsSeparatedByCharacters:@"\r\n"];
	//NSArray* commandArray = [tagFileContents componentsSeparatedByString:@"\r"];
	//NSLog([NSString stringWithFormat:@"command count = %d", [commandArray count]]);
	unsigned i;
	NSString* commandLine;
	NSMenu* menu = targetMenu;
	KMenuItem* menuItem;
	
	for(i = 0; i < [commandArray count]; i++){
		commandLine = [KStringUtils removeSpacesFromHeadOf:[commandArray objectAtIndex:i]];
		if([KStringUtils string:commandLine startsWith:@"//"]) { continue; }
		if([commandLine isEqualToString:@""]){ continue; }
		
		NSDictionary* dict = [KStringUtils resolveCommand:commandLine];
		//NSLog([NSString stringWithFormat:@"command = %@, string = %@", [dict objectForKey:@"command"], [dict objectForKey:@"string"]]);	
		if([[[dict objectForKey:@"command"] uppercaseString] isEqualToString:@"SHIFT"]){
			menuItem = [menu addItemWithTitle:[dict objectForKey:@"string"] action:nil keyEquivalent:@""];
			menu = [[NSMenu alloc] initWithTitle:@""];
			[menuItem setSubmenu:menu];
			
		}else if([[[dict objectForKey:@"command"] uppercaseString] isEqualToString:@"UNSHIFT"]){
			NSMenu* bufMenu = [menu supermenu];
			if(bufMenu){ menu = bufMenu; }
			
		}else if([[[dict objectForKey:@"command"] uppercaseString] isEqualToString:@"NAME"]){
			menuItem = [[KMenuItem alloc] initWithTitle:[dict objectForKey:@"string"] action:selector keyEquivalent:@""];
			[menuItem setAttachedObj:[NSMutableDictionary dictionaryWithCapacity:10]];
			[menu addItem:menuItem];
			
		}else if([[[dict objectForKey:@"command"] uppercaseString] isEqualToString:@"PRE"]){
			[[menuItem attachedObj] setObject:[dict objectForKey:@"string"] forKey:@"pre"];
		
		}else if([[[dict objectForKey:@"command"] uppercaseString] isEqualToString:@"POST"]){
			[[menuItem attachedObj] setObject:[dict objectForKey:@"string"] forKey:@"post"];
		
		}else if([[[dict objectForKey:@"command"] uppercaseString] isEqualToString:@"SINGLE"]){
			[[menuItem attachedObj] setObject:[dict objectForKey:@"string"] forKey:@"single"];
		
		}else if([[[dict objectForKey:@"command"] uppercaseString] isEqualToString:@"SELECT"]){
			[[menuItem attachedObj] setObject:[[dict objectForKey:@"string"] lowercaseString] forKey:@"select"];
		
		}else if([[[dict objectForKey:@"command"] uppercaseString] isEqualToString:@"MOD"]){
			NSString* mod = [dict objectForKey:@"string"];
			unsigned mask = NSCommandKeyMask;
			if([KStringUtils string:[mod uppercaseString] contains:@"OPT"]){ mask |= NSAlternateKeyMask; }
			if([KStringUtils string:[mod uppercaseString] contains:@"SHIFT"]){ mask |= NSShiftKeyMask; }
			if([KStringUtils string:[mod uppercaseString] contains:@"CTRL"]){ mask |= NSControlKeyMask; }
			[menuItem setKeyEquivalentModifierMask:mask];
		
		}else if([[[dict objectForKey:@"command"] uppercaseString] isEqualToString:@"KEY"]){
			NSString* key = [dict objectForKey:@"string"];
			if([[key uppercaseString] isEqualToString:@"RETURN"]){
				key = [NSString stringWithString:@"\r"];
			}else if([[key uppercaseString] isEqualToString:@"TAB"]){
				key = [NSString stringWithString:@"\t"];
			}else if([[key uppercaseString] isEqualToString:@"ESC"]){
				//key = [NSString stringWithString:@"\r"];
				//currently do nothing...
			}
			[menuItem setKeyEquivalent:[key lowercaseString]];
		}
	}
	
	/*
	NSFileManager* manager = [NSFileManager defaultManager];
	BOOL isDir;
	if(![manager fileExistsAtPath:dirName isDirectory:&isDir]){
		[manager createDirectoryAtPath:dirName attributes:nil];
	}
	if(![manager fileExistsAtPath:fileName]){
		
		[manager createFileAtPath:fileName contents:[NSData dataWithContentsOfFile:tagFileName] attributes:nil];
	}
	
	//NSMutableArray* funcArray = [[NSMutableArray alloc] initWithCapacity:100];
	NSArray* tagArray = nil;
	if(tagFileName){
		tagArray = [NSArray arrayWithContentsOfFile:tagFileName];
		[self constructTagMenu:targetMenu selector:selector tagArray:tagArray];
		[tagArray release];
	}*/
	
}

+ (void)attachTagMenuToMenuItem:(NSMenuItem*)targetMenuItem selector:(SEL)selector
{
	NSMenu* tagMenu = [[NSMenu alloc] initWithTitle:@"tag_submenu"];
	[targetMenuItem setSubmenu:tagMenu];
	[KMenuUtils attachTagMenuToMenu:tagMenu selector:selector];
}


+ (void)attachURLMenuToMenu:(NSMenu*)targetMenu selector:(SEL)selector
{
	// the format of dictionaries of array is below;
	// title : NSString : page title
	// url : NSString : page url
	
	////////
	// Internet Explorer Bookmarks
	////////
	NSMenuItem* menuItem = [[NSMenuItem alloc] initWithTitle:@"MSIE" action:nil keyEquivalent:@""];
	NSMenu* menu = [[NSMenu alloc] initWithTitle:@""];
	
	[targetMenu addItem:menuItem];
	[menuItem setSubmenu:menu];
	[KMenuUtils constructURLMenuWithIEBookmarks:menu selector:selector];
	
	
	////////
	// Safari Bookmarks
	////////
	menuItem = [[NSMenuItem alloc] initWithTitle:@"Safari" action:nil keyEquivalent:@""];
	menu = [[NSMenu alloc] initWithTitle:@""];
	[targetMenu addItem:menuItem];
	[menuItem setSubmenu:menu];
	[KMenuUtils constructURLMenuWithSafariBookmarks:menu selector:selector];

}

+ (void)attachURLMenuToMenuItem:(NSMenuItem*)targetMenuItem selector:(SEL)selector
{
	NSMenu* urlMenu = [[NSMenu alloc] initWithTitle:@"url_menu"];
	[targetMenuItem setSubmenu:urlMenu];
	
	return [self attachURLMenuToMenu:urlMenu selector:selector];
}

+ (void)attachScriptsMenuToMenu:(NSMenu*)targetMenu selector:(SEL)selector
{
	NSFileManager* manager = [NSFileManager defaultManager];
	NSString* addonsDirName = [[NSHomeDirectory() stringByAppendingPathComponent:KPreferenceDirectory]
									stringByAppendingPathComponent:KAddOnsDirectoryName];
	
	if(![manager fileExistsAtPath:addonsDirName]){
		[targetMenu addItemWithTitle:@"no item" action:nil keyEquivalent:@""];
		return;
	}
	[self constructScriptMenu:targetMenu dir:addonsDirName selector:(SEL)selector];
	
}

+ (void)attachScriptsMenuToMenuItem:(NSMenuItem*)targetMenuItem selector:(SEL)selector
{
	NSMenu* menu = [[NSMenu alloc] initWithTitle:@"no_title"];
	[self attachScriptsMenuToMenu:menu selector:selector];
	[targetMenuItem setSubmenu:menu];
}





// internal use only.
+ (void)constructScriptMenu:(NSMenu*)targetMenu dir:(NSString*)dirPath selector:(SEL)selector
{
	NSFileManager* manager = [NSFileManager defaultManager];
	//NSDirectoryEnumerator* enumerator = [manager enumeratorAtPath:dirPath];
	NSArray* contents = [manager directoryContentsAtPath:dirPath];
	NSString* filePath;
	NSString* fileName;
	NSString* key;
	KMenuItem* menuItem;
	NSMenu* menu;
	BOOL isDir;
	unsigned i;
	NSArray* components;
	
	for(i = 0; i < [contents count]; i++){
		fileName = [contents objectAtIndex:i];
		if([KStringUtils string:fileName startsWith:@"."]){ continue; }
		
		filePath = [dirPath stringByAppendingPathComponent:fileName];
		//NSLog(filePath);
		if([manager fileExistsAtPath:filePath isDirectory:&isDir]){
			fileName = [manager displayNameAtPath:filePath];
			components = [fileName componentsSeparatedByString:@"."];
			components = [[components objectAtIndex:0] componentsSeparatedByString:@"-"];
			
			menuItem = [[KMenuItem alloc] initWithTitle:fileName action:selector keyEquivalent:@""];
			[targetMenu addItem:menuItem];
			if(isDir){
				menu = [[NSMenu alloc] initWithTitle:@""];
				[menuItem setSubmenu:menu];
				[self constructScriptMenu:menu dir:filePath selector:selector];
			}else{
				[menuItem setAttachedObj:filePath];
				if([components count] >= 2){
					key = [components objectAtIndex:1];
					[menuItem setKeyEquivalent:key];
					[menuItem setKeyEquivalentModifierMask:(NSCommandKeyMask | NSAlternateKeyMask)];
				}
			}
		}
	}
	if(![targetMenu numberOfItems]){
		[targetMenu addItemWithTitle:@"no item" action:nil keyEquivalent:@""];
	}
}

+ (void)constructTagMenu:(NSMenu*)targetMenu selector:(SEL)selector tagArray:(NSArray*)tagArray
{
	KMenuItem* menuItem;
	NSMenu* subMenu;
	NSDictionary* dict;
	NSArray* subArray;
	NSString* title, *key, *modifiers;
	unsigned int modifiersMask = NSCommandKeyMask;
	unsigned i;
	for(i = 0; i < [tagArray count]; i++){
		dict = [tagArray objectAtIndex:i];
		title = [dict objectForKey:@"title"];
		key = [dict objectForKey:@"key"];
		modifiers = [dict objectForKey:@"modifiers"];
		
		if(!key){ key = [NSString stringWithString:@""]; }
		if([key isEqualToString:@"RETURN"]){
			key = [NSString stringWithString:@"\r"];
		}
		if([KStringUtils string:modifiers contains:@"option"]){
			modifiersMask |= NSAlternateKeyMask;
		}
		//modifiersMask &= ~NSShiftKeyMask;
		
		subArray = [dict objectForKey:@"sub_array"];
		menuItem = [[KMenuItem alloc] initWithTitle:title action:selector keyEquivalent:key];
		[menuItem setKeyEquivalentModifierMask:modifiersMask];
		if(subArray){
			subMenu = [[NSMenu alloc] initWithTitle:@""];
			[self constructTagMenu:subMenu selector:selector tagArray:subArray];
			[menuItem setSubmenu:subMenu]; 
		}else{
			[menuItem setAttachedObj:dict];
		}
		[targetMenu addItem:menuItem];
	}
	//NSLog([NSString stringWithFormat:@"constructTagMenu:count = %d", [tagArray count]]);
}


+ (void)constructURLMenuWithSafariBookmarks:(NSMenu*)targetMenu selector:(SEL)selector
{
	NSString* SafariBookmarkFileName = [NSHomeDirectory() stringByAppendingPathComponent:KSafariBookmarkFileName];
	NSDictionary* SafariBookmarkContents = [NSDictionary dictionaryWithContentsOfFile:SafariBookmarkFileName];
	if(!SafariBookmarkContents){ return; }
	NSArray* array = [SafariBookmarkContents objectForKey:@"Children"];
	if(!array){ return; }
	
	[KMenuUtils constructURLMenuWithSafariBookmarksWithArray:targetMenu selector:selector array:array];
	/*
	unsigned i;
	NSDictionary* dict;
	for(i = 0; i < [array count]; i++){
		dict = [array objectAtIndex:i];
		if([[dict objectForKey:@"Title"] isEqualToString:KSafariBookmarksMenuKey]){
			[KMenuUtils constructURLMenuWithSafariBookmarksWithArray:targetMenu selector:selector array:[dict objectForKey:@"Children"]];
			break;
		}
	}*/
}

+ (void)constructURLMenuWithSafariBookmarksWithArray:(NSMenu*)targetMenu selector:(SEL)selector array:(NSArray*)array
{
	unsigned i;
	NSDictionary* dict;
	NSArray* children = nil;
	NSMenu* menu;
	id menuItem;
	NSString* title, *url;
	
	for(i = 0; i < [array count]; i++){
		dict = [array objectAtIndex:i];
		if([[dict objectForKey:@"WebBookmarkType"] isEqualToString:@"WebBookmarkTypeList"]){
			if([[dict objectForKey:@"Title"] isEqualToString:@"Imported IE Favorites"]){ continue; }
			children = [dict objectForKey:@"Children"];
			if(children){
				menuItem = [[NSMenuItem alloc] initWithTitle:[dict objectForKey:@"Title"] action:nil keyEquivalent:@""];
				menu = [[NSMenu alloc] initWithTitle:@""];
				[targetMenu addItem:menuItem];
				[menuItem setSubmenu:menu];
				[KMenuUtils constructURLMenuWithSafariBookmarksWithArray:menu selector:selector array:children];
			}
		}else if([[dict objectForKey:@"WebBookmarkType"] isEqualToString:@"WebBookmarkTypeLeaf"]){
			title = [[dict objectForKey:@"URIDictionary"] objectForKey:@"title"];
			url = [dict objectForKey:@"URLString"];
			menuItem = [[KMenuItem alloc] initWithTitle:title action:selector keyEquivalent:@""];
			[targetMenu addItem:menuItem];
			[menuItem setAttachedObj:[NSDictionary dictionaryWithObjectsAndKeys:title, @"title", url, @"url", nil]];
		}
	}
}


+ (void)constructURLMenuWithIEBookmarks:(NSMenu*)targetMenu selector:(SEL)selector
{	
	NSString* IEBookmarkFileName = [NSHomeDirectory() stringByAppendingPathComponent:KIEBookmarkFileName];
	NSString* IEBookmarkContents = [NSString stringWithContentsOfFile:IEBookmarkFileName];
	
	if(IEBookmarkContents){
		NSArray* lineArray = [IEBookmarkContents componentsSeparatedByString:@"\r"];//[KStringUtils string:IEBookmarkContents separatedWith:@"\r"];
		NSArray* bufferArray;
		NSString* title, *url;
		int i;
		NSMenu* currentMenu = targetMenu;
		KMenuItem* currentMenuItem;
		
		for(i = 0; i < [lineArray count]; i++){
			if([KStringUtils string:[lineArray objectAtIndex:i] contains:@"</H3>"]){
				bufferArray = [KStringUtils string:[lineArray objectAtIndex:i] bindedWith:@"\">" andWith:@"</H3>"];
				if(!bufferArray || ![bufferArray count]){ continue; } // fixed with Mr. YOUsuke's advice. Thanx!

				currentMenuItem = [currentMenu addItemWithTitle:[bufferArray objectAtIndex:0] action:nil keyEquivalent:@""];
				currentMenu = [[NSMenu alloc] initWithTitle:@""];
				[currentMenuItem setSubmenu:currentMenu];
				continue;
			}else if([KStringUtils string:[lineArray objectAtIndex:i] contains:@"</DL>"]){
				if(currentMenu == targetMenu) { break; } // not allowed to touch the super menu of targetMenu.
				currentMenu = [currentMenu supermenu];
				continue;
			}
			bufferArray = [KStringUtils string:[lineArray objectAtIndex:i] bindedWith:@"A HREF=\"" andWith:@"\""];
			if([bufferArray count] > 0){
				url = [bufferArray objectAtIndex:0];
				bufferArray = [KStringUtils string:[lineArray objectAtIndex:i] bindedWith:@"\">" andWith:@"</A>"];
				if([bufferArray count] > 0){
					title = [bufferArray objectAtIndex:0];
					currentMenuItem = [[KMenuItem alloc] initWithTitle:title action:selector keyEquivalent:@""];
					[currentMenuItem setAttachedObj:[NSDictionary dictionaryWithObjectsAndKeys:title, @"title", url, @"url", nil]];
					[currentMenu addItem:currentMenuItem];
				}
			}
		}
	}
}


@end


////////
// KMenuItem
////////
@implementation KMenuItem

- (id)initWithTitle:(NSString*)itemName action:(SEL)anAction keyEquivalent:(NSString*)charCode
{
	mAttachedObj = nil;
	[super initWithTitle:itemName action:anAction keyEquivalent:charCode];
	return self;
}

- (void)dealloc
{
	[mAttachedObj release];
	[super dealloc];
}

- (id)attachedObj
{
	return mAttachedObj;
}

- (void)setAttachedObj:(id)obj
{
	[mAttachedObj release];
	mAttachedObj = [obj retain];
}

@end







