/* KPrefsServer */

#import <Cocoa/Cocoa.h>
#import "KColorUtils.h"

#define KKEditPrefs				@"kedit_prefs"

//#define KPreferenceDirectory @"Library/Preferences/KEdit"
#define KPreferenceDirectory @"Library/KEdit"
#define KAddOnsDirectoryName @"add-ons"
#define KSettingsDirectoryName @"settings"
#define KTagFileName @"UserDefinedTags.txt"

//BOOLs
#define KStoreWindowStatus		@"store_window_status"
#define KAutoIndent				@"auto_indent"
#define KWordWrap				@"word_wrap"
#define KDetectTextType			@"detect_text_type"
#define KDetectCodeType			@"detect_code_type"
#define KStoreTextType			@"store_text_type"
#define KAssignOnlyMultiStroke	@"assign_only_multi_stroke"
#define KUseXHTMLSyntax			@"use_xhtml_syntax"
#define KAlwaysMakesANewWindow	@"always_makes_a_new_window"

//INTs
#define KKeyBindType			@"key_bind_type"
#define KPreferredBrowser		@"preferred_browser"

//STRINGs
#define KFontName				@"font_name"

//FLOATs
#define KFontSize				@"font_size"

//COLORs
#define KBackgroundColor		@"background_color"
#define KBaseStringColor		@"base_string_color"
#define KTagStringColor			@"tab_string_color"
#define KCommentStringColor		@"comment_string_color"


typedef enum {
	KKEditOriginal = 1,
	KEmacsLike = 2,
	KViLike = 3,
	KSystemDefault = 4
} KKeyBind;

typedef enum {
	KInternetExplorer = 1,
	KSafari = 2
} KBrowser;


@interface KPrefsServer : NSObject
{
	IBOutlet id myWindow;
	IBOutlet id myTabView;
	
	// "General"
	IBOutlet id myAutoIndentCheckBox;
	IBOutlet id myWordWrapCheckBox;
	IBOutlet id myCharCodeDetectionCheckBox;
	IBOutlet id myFileTypeDetectionCheckBox;
	IBOutlet id myStoreWindowStatusCheckBox;
	IBOutlet id myUseXHTMLSyntaxCheckBox;
	IBOutlet id myAlwaysMakesANewWindowCheckBox;
	IBOutlet id myBrowserMenu;
	
	// "Key Assign"
	IBOutlet id myKeyAssignMenu;
	IBOutlet id myAssignOnlyMultiStrokeKeyBindCheckBox;
	
	// "Font & Color"
	IBOutlet id myFontFamilyMenu;
	IBOutlet id myFontSizeMenu;
	
	IBOutlet id myBaseColorWell;
	IBOutlet id myBackgroudColorWell;
	IBOutlet id myTagFontColorWell;
	IBOutlet id myCommentFontColorWell;
	
	NSMutableDictionary* mPrefsDictionary;
	NSMutableDictionary* mTempDictionary;
	
	NSArray* mFontNameArray;
	
	// "Others"
	// do nothing...
		
}


- (void)showPreferenceWindow;

//actions.
- (IBAction)autoIndentCheckBoxPressed:(id)sender;
- (IBAction)wordWrapCheckBoxPressed:(id)sender;
- (IBAction)charCodeDetectionCheckBoxPressed:(id)sender;
- (IBAction)fileTypeDetectionCheckBoxPressed:(id)sender;
- (IBAction)storeWindowStatusCheckBoxPressed:(id)sender;
- (IBAction)useXHTMLSyntaxCheckBoxPressed:(id)sender;
- (IBAction)alwaysMakesANewWindowCheckBoxPressed:(id)sender;
- (IBAction)browserMenuSelected:(id)sender;

- (IBAction)keyAssignMenuSelected:(id)sender;
- (IBAction)assignOnlyMultiStrokeKeyBindCheckBoxPressed:(id)sender;

- (IBAction)fontFamilyMenuSelected:(id)sender;
- (IBAction)fontSizeMenuSelected:(id)sender;
// currently color wells are no function.
- (IBAction)okButtonPressed:(id)sender;
- (IBAction)cancelButtonPressed:(id)sender;

//access prefs.
- (id)objectForKey:(id)key;
- (BOOL)boolForKey:(id)key;
- (float)floatForKey:(id)key;
- (int)intForKey:(id)key;
- (rgb_color)colorForKey:(id)key;

//utility methods.
//- (NSFont*)textViewFont;
- (NSColor*)nscolorForKey:(id)key;

//for internal use only.
- (void)loadPreferenceWindow;
- (BOOL)windowShouldClose:(id)sender;

- (void)resetUI;
- (void)dealloc;

@end


@interface NSMutableDictionary (Preferences)

- (BOOL)boolForKey:(id)key;
- (void)setBool:(BOOL)flag forKey:(id)key;
- (float)floatForKey:(id)key;
- (void)setFloat:(float)number forKey:(id)key;
- (int)intForKey:(id)key;
- (void)setInt:(int)number forKey:(id)key;
- (rgb_color)colorForKey:(id)key;
- (void)setColor:(rgb_color)color forKey:(id)key;

@end












