#import "KPrefsServer.h"
#import "KColorUtils.h"
#import "KMenuUtils.h"
#import "KApplicationDel.h"

@implementation KPrefsServer

- (id)init
{
	mFontNameArray = nil;
	
	NSString* fontName = [[NSFont userFixedPitchFontOfSize:0.0] fontName];
	rgb_color backgroundColor = { 1.0, 1.0, 1.0, 1.0 };
	rgb_color basestringColor = { 0.0, 0.0, 0.0, 1.0 };
	rgb_color tagstringColor = { 0.39, 0.39, 0.78, 1.0 };
	rgb_color commentstringColor = { 0.67, 0.0, 0.0, 1.0 };
	
	mPrefsDictionary = [[NSMutableDictionary alloc] initWithCapacity:100];
	
	[mPrefsDictionary setBool:YES forKey:KStoreWindowStatus];
	[mPrefsDictionary setBool:YES forKey:KAutoIndent];
	[mPrefsDictionary setBool:YES forKey:KWordWrap];
	[mPrefsDictionary setBool:YES forKey:KDetectTextType];
	[mPrefsDictionary setBool:YES forKey:KDetectCodeType];
	[mPrefsDictionary setBool:YES forKey:KStoreTextType];
	[mPrefsDictionary setBool:NO forKey:KAssignOnlyMultiStroke];
	[mPrefsDictionary setBool:YES forKey:KUseXHTMLSyntax];
	[mPrefsDictionary setBool:NO forKey:KAlwaysMakesANewWindow];
	[mPrefsDictionary setInt:(int)KInternetExplorer forKey:KPreferredBrowser];
	
	[mPrefsDictionary setInt:(int)KKEditOriginal forKey:KKeyBindType];
	[mPrefsDictionary setObject:fontName forKey:KFontName];
	[mPrefsDictionary setFloat:12.0 forKey:KFontSize];
	
	
	[mPrefsDictionary setColor:backgroundColor forKey:KBackgroundColor];
	[mPrefsDictionary setColor:basestringColor forKey:KBaseStringColor];
	[mPrefsDictionary setColor:tagstringColor forKey:KTagStringColor];
	[mPrefsDictionary setColor:commentstringColor forKey:KCommentStringColor];
	
	//set prefs.
	NSUserDefaults* def = [NSUserDefaults standardUserDefaults];
	
	NSDictionary* pastPrefs = [def objectForKey:KKEditPrefs];
	if(!pastPrefs){
		[def setObject:mPrefsDictionary forKey:KKEditPrefs];
	}else{
		[mPrefsDictionary addEntriesFromDictionary:pastPrefs];
		[def setObject:mPrefsDictionary forKey:KKEditPrefs];
	}
	mTempDictionary = [[NSMutableDictionary alloc] initWithDictionary:mPrefsDictionary copyItems:YES];
	//[def synchronize]; 
	
	return self;
}

- (void)dealloc
{
	
	//NSUserDefaults* def = [NSUserDefaults standardUserDefaults];
	
	[mPrefsDictionary release];
	[mTempDictionary release];
	
	NSLog(@"KPrefsServer : dealloc - done");
	
	[super dealloc];
}


- (void)showPreferenceWindow
{
	if(myWindow == nil){
		[self loadPreferenceWindow];
	}
	[myWindow makeKeyAndOrderFront:self];
}

- (void)loadPreferenceWindow
{
	[NSBundle loadNibNamed:@"PreferenceWindow" owner:self];
}


// Panel of PreferenceWindow.nib : delegate method
- (void)awakeFromNib
{
	//sync checks for prefs.
	//Add FontFamily&Face Menu.
	NSMenu* fontMenu = [[NSMenu alloc] initWithTitle:@""];
	mFontNameArray = [KMenuUtils attachFlattedFontMenuToMenu:fontMenu selector:@selector(fontFamilyMenuSelected:)];
	[myFontFamilyMenu setMenu:fontMenu];
	
	//Add FontSize Menu.
	NSMenu* sizeMenu = [[NSMenu alloc] initWithTitle:@""];
	NSArray* array = [KMenuUtils attachSizeMenuToMenu:sizeMenu selector:@selector(fontSizeMenuSelected:)];
	[myFontSizeMenu setMenu:sizeMenu];
	[array release];
	
	[myTabView selectFirstTabViewItem:self];
	[self resetUI];

}

- (void)resetUI
{
	// "General"
	[myAutoIndentCheckBox setState:[mPrefsDictionary boolForKey:KAutoIndent]];
	[myWordWrapCheckBox setState:[mPrefsDictionary boolForKey:KWordWrap]];
	[myCharCodeDetectionCheckBox setState:[mPrefsDictionary boolForKey:KDetectCodeType]];
	[myFileTypeDetectionCheckBox setState:[mPrefsDictionary boolForKey:KDetectTextType]];
	[myStoreWindowStatusCheckBox setState:[mPrefsDictionary boolForKey:KStoreWindowStatus]];
	[myUseXHTMLSyntaxCheckBox setState:[mPrefsDictionary boolForKey:KUseXHTMLSyntax]];
	NSMenuItem* browserMenu = [KMenuUtils searchMenuItemWithTag:[mPrefsDictionary intForKey:KPreferredBrowser] targetMenu:[myBrowserMenu menu]];
	[myBrowserMenu selectItem:browserMenu];
	
	// "Key Assign"
	[myAssignOnlyMultiStrokeKeyBindCheckBox setState:[mPrefsDictionary boolForKey:KAssignOnlyMultiStroke]];
	// KeyBindMenu update.
	NSMenuItem* assignMenu = [KMenuUtils searchMenuItemWithTag:[mPrefsDictionary intForKey:KKeyBindType] targetMenu:[myKeyAssignMenu menu]];
	[myKeyAssignMenu selectItem:assignMenu];
	//NSLog([NSString stringWithFormat:@"tag = %d", [mPrefsDictionary intForKey:KKeyBindType]]);
	
	// "Font & Color"
	// FontFamilyMenu update.
	int tag = [mFontNameArray indexOfObject:[mPrefsDictionary objectForKey:KFontName]];
	NSMenuItem* fontMenuItem = [KMenuUtils searchMenuItemWithTag:tag targetMenu:[myFontFamilyMenu menu]];
	[myFontFamilyMenu selectItem:fontMenuItem];
	
	// FontSizeMenu update.
	NSArray* sizeArray = [[NSApp delegate] menuDict:KFontSizeArray];
	tag = [sizeArray indexOfObject:[NSNumber numberWithFloat:[mPrefsDictionary floatForKey:KFontSize]]];
	NSMenuItem* sizeMenuItem = [KMenuUtils searchMenuItemWithTag:tag targetMenu:[myFontSizeMenu menu]];
	[myFontSizeMenu selectItem:sizeMenuItem];
	
	// ColorWells update.
	[myBaseColorWell setColor:[self nscolorForKey:KBaseStringColor]];
	[myBackgroudColorWell setColor:[self nscolorForKey:KBackgroundColor]];
	[myTagFontColorWell setColor:[self nscolorForKey:KTagStringColor]];
	[myCommentFontColorWell setColor:[self nscolorForKey:KCommentStringColor]];
}

- (BOOL)windowShouldClose:(id)sender
{
	if(sender == myWindow){
		[myWindow orderOut:self];
		[mTempDictionary release];
		mTempDictionary = [[NSMutableDictionary alloc] initWithDictionary:mPrefsDictionary copyItems:YES];
		[self resetUI];
		return NO;
	}
	return YES;
}


////////
// Actions
////////

- (IBAction)autoIndentCheckBoxPressed:(id)sender
{
	[mTempDictionary setBool:(BOOL)[sender state] forKey:KAutoIndent];
	//[sender setState:![sender state]];
}

- (IBAction)wordWrapCheckBoxPressed:(id)sender
{
	[mTempDictionary setBool:(BOOL)[sender state] forKey:KWordWrap];
}

- (IBAction)charCodeDetectionCheckBoxPressed:(id)sender
{
	[mTempDictionary setBool:(BOOL)[sender state] forKey:KDetectCodeType];
}

- (IBAction)fileTypeDetectionCheckBoxPressed:(id)sender
{
	[mTempDictionary setBool:(BOOL)[sender state] forKey:KDetectTextType];
}

- (IBAction)storeWindowStatusCheckBoxPressed:(id)sender
{
	[mTempDictionary setBool:(BOOL)[sender state] forKey:KStoreWindowStatus];
}

- (IBAction)useXHTMLSyntaxCheckBoxPressed:(id)sender
{
	[mTempDictionary setBool:(BOOL)[sender state] forKey:KUseXHTMLSyntax];
}

- (IBAction)alwaysMakesANewWindowCheckBoxPressed:(id)sender
{
	[mTempDictionary setBool:(BOOL)[sender state] forKey:KAlwaysMakesANewWindow];
}

- (IBAction)browserMenuSelected:(id)sender
{
	[mTempDictionary setInt:[sender tag] forKey:KPreferredBrowser];
	//NSLog([NSString stringWithFormat:@"tag = %d", [sender tag]]);	
}

- (IBAction)keyAssignMenuSelected:(id)sender
{
	[mTempDictionary setInt:[sender tag] forKey:KKeyBindType];
	//NSLog([NSString stringWithFormat:@"selected tag = %d(%@)", [sender tag], [sender title]]);
}

- (IBAction)assignOnlyMultiStrokeKeyBindCheckBoxPressed:(id)sender
{
	[mTempDictionary setBool:(BOOL)[sender state] forKey:KAssignOnlyMultiStroke];
}

- (IBAction)fontFamilyMenuSelected:(id)sender
{
	[mTempDictionary setObject:[mFontNameArray objectAtIndex:[sender tag]] forKey:KFontName];
}

- (IBAction)fontSizeMenuSelected:(id)sender
{
	[mTempDictionary setObject:[[[NSApp delegate] menuDict:KFontSizeArray] objectAtIndex:[sender tag]] forKey:KFontSize];
}

- (IBAction)okButtonPressed:(id)sender
{
	[myWindow orderOut:self];
	
	[mTempDictionary setColor:[KColorUtils convertNSColorToStruct:[myBaseColorWell color]] forKey:KBaseStringColor];
	[mTempDictionary setColor:[KColorUtils convertNSColorToStruct:[myBackgroudColorWell color]] forKey:KBackgroundColor];
	[mTempDictionary setColor:[KColorUtils convertNSColorToStruct:[myTagFontColorWell color]] forKey:KTagStringColor];
	[mTempDictionary setColor:[KColorUtils convertNSColorToStruct:[myCommentFontColorWell color]] forKey:KCommentStringColor];

	[mPrefsDictionary release];
	mPrefsDictionary = [[NSMutableDictionary alloc] initWithDictionary:mTempDictionary copyItems:YES];
	
	[[NSUserDefaults standardUserDefaults] setObject:mPrefsDictionary forKey:KKEditPrefs];
	
	[self resetUI];
}

- (IBAction)cancelButtonPressed:(id)sender
{
	[myWindow orderOut:self];
	[mTempDictionary release];
	mTempDictionary = [[NSMutableDictionary alloc] initWithDictionary:mPrefsDictionary copyItems:YES];
	
	[self resetUI];
}




// Access prefs.

- (id)objectForKey:(id)key { return [mPrefsDictionary objectForKey:key]; }
- (BOOL)boolForKey:(id)key { return [mPrefsDictionary boolForKey:key]; }
- (float)floatForKey:(id)key { return [mPrefsDictionary floatForKey:key]; }
- (int)intForKey:(id)key { return [mPrefsDictionary intForKey:key]; }
- (rgb_color)colorForKey:(id)key { return [mPrefsDictionary colorForKey:key]; }

- (NSColor*)nscolorForKey:(id)key { return [KColorUtils convertStructToNSColor:[mPrefsDictionary colorForKey:key]]; }
//- (NSFont*)textViewFont { return [NSFont fontWithName:[mPrefsDictionary objectForKey:KFontName] size:[mPrefsDictionary objectForKey:KFontSize]]; }



@end



// NSMutableDictionary.
@implementation NSMutableDictionary (Preferences)

- (BOOL)boolForKey:(id)key { return [[self objectForKey:key] boolValue]; }
- (void)setBool:(BOOL)flag forKey:(id)key { [self setObject:[NSNumber numberWithBool:flag] forKey:key]; }
- (float)floatForKey:(id)key { return [[self objectForKey:key] floatValue]; }
- (void)setFloat:(float)number forKey:(id)key { [self setObject:[NSNumber numberWithFloat:number] forKey:key]; }
- (int)intForKey:(id)key { return [[self objectForKey:key] intValue]; }
- (void)setInt:(int)number forKey:(id)key { [self setObject:[NSNumber numberWithInt:number] forKey:key]; }
- (rgb_color)colorForKey:(id)key { return *(rgb_color*)[[self objectForKey:key] bytes]; }
- (void)setColor:(rgb_color)color forKey:(id)key { [self setObject:[NSData dataWithBytes:&color length:sizeof(rgb_color)] forKey:key]; }

@end










