#import "KTextView.h"
#import "KTextUtils.h"
#import "KStringUtils.h"
#import "KKeyAssignServer.h"
#import "KApplicationDel.h"
#import "KPrefsServer.h"
#import "KMenuUtils.h"
#import "KColorUtils.h"
#import "KRangeObj.h"

#import "KSettingFile.h"


@implementation KTextView

- (id)initWithFrame:(NSRect)rect
{
	self = [super initWithFrame:rect];
	
	if(self){
		[self registerForDraggedTypes:[NSArray arrayWithObjects:NSFilenamesPboardType, nil]];
		[self setAllowsUndo:YES];
		
		mKeyAssignServer = [[KKeyAssignServer alloc] init];
		//NSLog(@"init");
	}
	
	return self;
}

- (void)dealloc
{
	[mKeyAssignServer release];
}

- (void)keyDown:(NSEvent*)theEvent
{
	KPrefsServer* prefs = [[NSApp delegate] prefsServer];
	
	if(![self hasMarkedText] && ([prefs intForKey:KKeyBindType] != KSystemDefault)){
		if(![mKeyAssignServer doneAssignedFunction:theEvent sender:self]){
			[super keyDown:theEvent];
		}
	}else{
		//While IM input.
		[super keyDown:theEvent];
	}
}

- (void)insertText:(id)aString
{
	//NSString* string = [KStringUtils convertReturnsToCR:aString from:nil];
	//NSLog(@"insertText");
	[super insertText:aString];
}

- (NSMenu*)menuForEvent:(NSEvent*)event
{
	NSMenu* menu = [[NSMenu alloc] initWithTitle:@""];
	NSMenuItem* menuItem;
	
	[menu addItemWithTitle:NSLocalizedString(@"Cut", @"") action:@selector(cut:) keyEquivalent:@""];
	[menu addItemWithTitle:NSLocalizedString(@"Copy", @"") action:@selector(copy:) keyEquivalent:@""];
	[menu addItemWithTitle:NSLocalizedString(@"Paste", @"") action:@selector(paste:) keyEquivalent:@""];
	[menu addItemWithTitle:NSLocalizedString(@"Select All", @"") action:@selector(selectAll:) keyEquivalent:@""];
	
	[menu addItem:[NSMenuItem separatorItem]];
	
	[menu addItemWithTitle:NSLocalizedString(@"Information", @"") action:@selector(showInformationDialog:) keyEquivalent:@""];
	
	[menu addItem:[NSMenuItem separatorItem]];
	
	menuItem = [menu addItemWithTitle:NSLocalizedString(@"HTML Tags", @"") action:nil keyEquivalent:@""];
	[KMenuUtils attachTagMenuToMenuItem:menuItem selector:@selector(insertMenuTag:)];
	menuItem = [menu addItemWithTitle:NSLocalizedString(@"URLs", @"") action:nil keyEquivalent:@""];
	[KMenuUtils attachURLMenuToMenuItem:menuItem selector:@selector(insertMenuURL:)];
	menuItem = [menu addItemWithTitle:NSLocalizedString(@"Tools", @"") action:nil keyEquivalent:@""];
	[KMenuUtils attachScriptsMenuToMenuItem:menuItem selector:@selector(executeScript:)];
	
	return menu;
}

- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender
{
	return [self draggingUpdated:sender];
}

- (NSDragOperation)draggingUpdated:(id <NSDraggingInfo>)sender
{
	//NSLog(@"draggingUpdated.");
	NSPasteboard *pboard;
    NSDragOperation sourceDragMask;
	
    sourceDragMask = [sender draggingSourceOperationMask];
    pboard = [sender draggingPasteboard];
	
    if ( [[pboard types] containsObject:NSColorPboardType] ) {
        if (sourceDragMask & NSDragOperationCopy) {
            return NSDragOperationCopy;
        }
    }
    if ( [[pboard types] containsObject:NSFilenamesPboardType] ) {
		//NSLog(@"draggingEntered - filenames");
        if (sourceDragMask & NSDragOperationLink) {
            return NSDragOperationLink;
        } else if (sourceDragMask & NSDragOperationCopy) {
            return NSDragOperationCopy;
        }
		//NSLog(@"draggingEntered - else");
    }
    return [super draggingEntered:sender];
	
}

- (BOOL)prepareDragOperation:(id <NSDraggingInfo>)sender
{
	//NSLog(@"prepareDragOperation");
	return YES;
}

- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender
{
    return YES;
}

- (void)concludeDragOperation:(id <NSDraggingInfo>)sender
{
	NSPasteboard* pboard = [sender draggingPasteboard];
    NSDragOperation sourceDragMask =  [sender draggingSourceOperationMask];
	
	//NSLog([NSString stringWithFormat:@"dinfo name = %@", [pboard name]]);

    if ( [[pboard types] containsObject:NSColorPboardType] ) {
		//NSLog(@"color is dropped");
		[self insertColorString:[NSColor colorFromPasteboard:pboard]];
        // Only a copy operation allowed so just copy the data
        //NSColor *newColor = [NSColor colorFromPasteboard:pboard];
       // [self setColor:newColor];
    } else if ( [[pboard types] containsObject:NSFilenamesPboardType] ) {
        NSArray* files = [pboard propertyListForType:NSFilenamesPboardType];
        // Depending on the dragging source and modifier keys,
        // the file data may be copied or linked
        if (sourceDragMask & NSDragOperationLink) {
			//do nothing or controll key pressed.
			//load the text of dropped files to selection
			//NSLog(@"perform:LINK");
			unsigned count = [files count];
			unsigned i;
			NSString* fileName;
			for(i = 0; i < count; i++){
				fileName = [files objectAtIndex:i];
				
				NSFileWrapper* aFile = [[NSFileWrapper alloc] initWithPath:fileName];
				NSDictionary* dict = [aFile fileAttributes];
				NSNumber* creatorCode = [dict objectForKey:NSFileHFSCreatorCode];
				NSNumber* typeCode = [dict objectForKey:NSFileHFSTypeCode];
				
				if([creatorCode isEqualToNumber:[NSNumber numberWithUnsignedLong:'MACS']]
						&& [typeCode isEqualToNumber:[NSNumber numberWithUnsignedLong:'clpt']]){
					// Text Clipping File.
					[super concludeDragOperation:sender];
				
				}else if([[fileName pathExtension] isEqualToString:@"jpg"]
					|| [[fileName pathExtension] isEqualToString:@"png"]
					|| [[fileName pathExtension] isEqualToString:@"gif"]){
					// treat IMAGE files.
					[self insertImageTag:fileName];
				
				}else{
					// treat non-IMAGE files.
					[self insertTextOfFile:fileName];
				}
				[aFile release];
			}
		}else {
			//option key pressed.
			//set selection to the relative path name.
			//NSLog(@"perform:COPY");
			[self insertRelativePath:[files objectAtIndex:0]];
        }
	}else if([[pboard types] containsObject:NSURLPboardType]){
		[self insertURL:[[pboard propertyListForType:NSURLPboardType] objectAtIndex:0]];
    }else if([[pboard types] containsObject:NSStringPboardType]){
		//NSLog(@"String!");
		
		KReturnCodeType cType;
		//NSRange range = [self selectedRange];
		//NSLog([pboard propertyListForType:NSStringPboardType]);
		// NSStringFromClass([pString class]);
		NSString* pString = [pboard stringForType:NSStringPboardType];
		pString = [KStringUtils convertReturnsToCR:pString from:&cType];
		BOOL isOK = [pboard setString:pString forType:NSStringPboardType];
		NSLog([NSString stringWithFormat:@"isOK = %d", isOK]);
		/*
		NSPoint insPoint = [sender draggingLocation];
		insPoint = [[self window] convertBaseToScreen:insPoint];
		NSLog(NSStringFromPoint(insPoint));
		
		unsigned charIndex = [self characterIndexForPoint:insPoint];
		NSLog([NSString stringWithFormat:@"index = %d", charIndex]);
		[self setSelectedRange:NSMakeRange(charIndex, 0)];
		[self insertText:pString];
		[self setSelectedRange:NSMakeRange(charIndex, [pString length])];*/
		
		[super concludeDragOperation:sender];
		
	}else{
		[super concludeDragOperation:sender];
	}
}

- (void)insertTextOfFile:(NSString*)fileName
{
	NSFileWrapper* wrapper = [[NSFileWrapper alloc] initWithPath:fileName];
	NSData* data = [wrapper regularFileContents];

	if(![data length]){
		NSLog(@"KTextView insertTextOfFile: - data length == 0");
		[wrapper release];
		return;
	}
	//NSLog([NSString stringWithFormat:@"insertTextOfFile: - %d", [data length]]);
	NSStringEncoding encoding = [KTextUtils detectCharacterCodeType:data];
	NSString* string = [[NSString alloc] initWithData:data encoding:encoding];
	
	
	NSRange range = [self selectedRange];
	[self insertText:string];
	[self setSelectedRange:NSMakeRange(range.location, range.length + [string length])];
	[self scrollRangeToVisible:[self selectedRange]];
	[string release];
	[wrapper release];
}

- (void)insertRelativePath:(NSString*)fileName
{
	NSRange selectedRange = [self selectedRange];
	NSString* documentName = [myDocument fileName];
	if(documentName == nil){ [self insertText:fileName]; return; }
	
	NSString* newString = [KStringUtils relativePathFrom:documentName to:fileName];
		
	if(selectedRange.length == 0){
		[self insertText:newString];
	}else{
		NSString* string = [NSString stringWithFormat:@"<a href=\"%@\">", newString];
		[self insertTagsWithPrefix:string suffix:@"</a>"];
	}
}

- (void)insertTagsWithPrefix:(NSString*)prefix suffix:(NSString*)suffix
{
	[self insertTagsWithPrefix:prefix suffix:suffix single:nil insertion:KSelectWholeInsertionPoint];
}

- (void)insertTagsWithPrefix:(NSString*)prefix suffix:(NSString*)suffix single:(NSString*)single insertion:(KInsertedSelectionType)type
{
	NSRange selectedRange = [self selectedRange];
	NSString* string;
	if((selectedRange.length == 0) && single){
		string = single;
	}else{
		string = [NSString stringWithFormat:@"%@%@%@", prefix, [[self string] substringWithRange:selectedRange], suffix];
	}
	[self insertText:string];
	if(type == KSelectBeginingOfInsersionPoint){
		[self setSelectedRange:NSMakeRange(selectedRange.location, 0)];
	}else if(type == KSelectEndOfInsertionPoint){
		[self setSelectedRange:NSMakeRange(selectedRange.location + [string length], 0)];
	}else if(type == KSelectWholeInsertionPoint){
		[self setSelectedRange:NSMakeRange(selectedRange.location, [string length])];
	}else if(type == KSelectInsideInsertionPoint){
		[self setSelectedRange:NSMakeRange(selectedRange.location + [prefix length], selectedRange.length)];
	}
	
	[self scrollRangeToVisible:[self selectedRange]];
}

- (void)selectLineTopOf:(unsigned)lineIndex
{
	NSArray* lineRangeArray = [KStringUtils rangesOfLines:[self string] range:NSMakeRange(0, [[self string] length])];
	if((lineIndex > [lineRangeArray count]) || (lineIndex < 1)) {
		NSLog(@"over-range of lines.");
		NSBeep();
		return;
	}
	NSRange range = NSMakeRange([[lineRangeArray objectAtIndex:(lineIndex - 1)] range].location, 0);
	[self setSelectedRange:range];
	[self scrollRangeToVisible:range];
}

- (void)selectLineOf:(unsigned)lineIndex
{
	NSArray* lineRangeArray = [KStringUtils rangesOfLines:[self string] range:NSMakeRange(0, [[self string] length])];
	if((lineIndex > [lineRangeArray count]) || (lineIndex < 1)) {
		NSLog(@"over-range of lines.");
		NSBeep();
		return;
	}
	NSRange range = [[lineRangeArray objectAtIndex:(lineIndex - 1)] range];
	[self setSelectedRange:range];
	[self scrollRangeToVisible:range];
}

- (void)insertImageTag:(NSString*)fileName
{
	NSString* documentName = [myDocument fileName];
	NSString* relativePath;
	if(documentName == nil){
		relativePath = fileName;
	}else{
		relativePath = [KStringUtils relativePathFrom:[myDocument fileName] to:fileName];
	}
	
	
	NSArray* array = [NSImageRep imageRepsWithContentsOfFile:fileName];
	
	if(array){
		NSImageRep* rep = [array objectAtIndex:0];
		int height = [rep pixelsHigh];
		int width = [rep pixelsWide];
		NSString* string;
		
		KPrefsServer* prefs = [[NSApp delegate] prefsServer];
		if([prefs boolForKey:KUseXHTMLSyntax]){
			string = [NSString stringWithFormat:@"<img src=\"%@\" width=\"%d\" height=\"%d\" alt=\"\" />", relativePath, width, height];
		}else{
			string = [NSString stringWithFormat:@"<img src=\"%@\" width=\"%d\" height=\"%d\" alt=\"\">", relativePath, width, height];
		}
		NSRange selectedRange = [self selectedRange];
		[self insertText:string];
		[self setSelectedRange:NSMakeRange(selectedRange.location, [string length])];
		[self scrollRangeToVisible:[self selectedRange]];
	}
}

- (void)insertURL:(NSString*)urlString
{
	NSRange selectedRange = [self selectedRange];
	if(selectedRange.length == 0){
		[self insertText:urlString];
		[self setSelectedRange:NSMakeRange(selectedRange.location, [urlString length])];
	}else{
		[self insertTagsWithPrefix:[NSString stringWithFormat:@"<a href=\"%@\">", urlString] suffix:@"</a>"];
	}
}

- (void)insertColorString:(NSColor*)color
{
	NSString* string = [KStringUtils stringFromColor:color];
	NSRange range = [self selectedRange];
	
	[self insertText:string];
	[self setSelectedRange:NSMakeRange(range.location, [string length])];
}


////////
// additional key assigns.
////////
- (void)moveForwardAndModifySelection_kedit:(id)sender
{
	NSRange selectedRange = [self selectedRange];
	if(NSMaxRange(selectedRange) < [[self string] length]){
		selectedRange.length++;
		[self setSelectedRange:selectedRange];
	}
}

- (void)moveBackwardAndModifySelection_kedit:(id)sender
{
	NSRange selectedRange = [self selectedRange];
	if(selectedRange.location > 0){
		selectedRange.location--;
		selectedRange.length++;
		[self setSelectedRange:selectedRange];
	}
}

- (void)moveToBeginningOfParagraphAndModifySelection:(id)sender
{
	NSRange selectedRange = [self selectedRange];
	NSRange lineRange = [[self string] lineRangeForRange:selectedRange];
	selectedRange.length += selectedRange.location - lineRange.location;
	selectedRange.location = lineRange.location;
	[self setSelectedRange:selectedRange];
}

- (void)moveToEndOfParagraphAndModifySelection:(id)sender
{
	NSRange selectedRange = [self selectedRange];
	NSRange lineRange = [[self string] lineRangeForRange:selectedRange];
	selectedRange.length = lineRange.length - selectedRange.location + lineRange.location;
	[self setSelectedRange:selectedRange];
	
}

- (void)doSample:(id)sender
{
	NSLog(@"do sample");
	NSString* target = [self string];
	[KStringUtils countReturns:target];
	
	kunistr str1 = uni_construct_with_string(@"abcd");
	unichar uc;
	[@"e" getCharacters:&uc];
	int offset = uni_offset_of_char(str1, uc);
	NSLog([NSString stringWithFormat:@"offset = %d", offset]);
	
	/*kunistr str2 = uni_construct_with_string(@"0123");
	uni_append_str(&str1, &str2);
	uni_print_to_stream(str1);
	uni_destruct(str2);
	uni_destruct(str1);*/
}


- (void)autoIndent:(id)sender // called by KKeyAssignServer.
{	
	NSRange selectedRange = [self selectedRange];
	NSRange lineRange = [[self string] lineRangeForRange:selectedRange];
	
	if(([[self string] length] < 1) || (selectedRange.location == lineRange.location)){
		[self insertText:@"\r"];
		return;
	}
	
	NSMutableString* string = [[NSMutableString alloc] initWithString:@""];
	NSString* substring;
	int i;
	
	for(i = 0; i < lineRange.length; i++){
		substring = [[self string] substringWithRange:NSMakeRange(lineRange.location + i, 1)];
		if([substring isEqualToString:@"\t"] || [substring isEqualToString:@" "]){
			[string appendString:substring];
		}else{
			break;
		}
	}
	[self insertText:KRETURN_char];
	[self insertText:string];
	
	[string release];
}

- (IBAction)insertMenuTag:(id)sender
{
	NSDictionary* dict = [sender attachedObj];
	NSString* pre = [dict objectForKey:@"pre"];
	NSString* post = [dict objectForKey:@"post"];
	NSString* single = [dict objectForKey:@"single"];
	NSString* typestr = [dict objectForKey:@"select"];
	KInsertedSelectionType type = KSelectWholeInsertionPoint;
	
	if(!pre){
		pre = [NSString stringWithString:@""];
	}
	if(!post){
		post = [NSString stringWithString:@""];
	}
	if(!single){
		//NSLog(@"single == nil");
	}
	if(!typestr){
		//type = KSelectWholeInsertionPoint;
	}else{
		if([typestr isEqualToString:@"pre"]){
			type = KSelectBeginingOfInsersionPoint;
		}else if([typestr isEqualToString:@"post"]){
			type = KSelectEndOfInsertionPoint;
		}else if([typestr isEqualToString:@"all"]){
			type = KSelectWholeInsertionPoint;
		}else if([typestr isEqualToString:@"inside"]){
			type = KSelectInsideInsertionPoint;
		}
	}
	[self insertTagsWithPrefix:pre suffix:post single:single insertion:type];
}

- (IBAction)insertMenuURL:(id)sender
{
	//NSLog(@"insertMenuURL");
	//NSDictionary* urlDict = [[[NSApp delegate] menuDict:KURLArray] objectAtIndex:[sender tag]];
	NSDictionary* urlDict = [sender attachedObj];
	NSRange selectedRange = [self selectedRange];
	if(selectedRange.length == 0){
		[self insertTagsWithPrefix:[urlDict objectForKey:@"url"] suffix:@""];
	}else{
		//[NSString stringWithFormat:@"<A HREF=\"%@\">", [urlDict objectForKey:@"url"]]
		[self insertTagsWithPrefix:[NSString stringWithFormat:@"<a href=\"%@\">", [urlDict objectForKey:@"url"]] suffix:@"</a>"];
	}
}


- (IBAction)wrapWithReturn:(id)sender
{
	NSRange selectedRange = [self selectedRange];
	selectedRange = [[self string] lineRangeForRange:selectedRange];
	[self setSelectedRange:selectedRange];
	NSString* target = [[self string] substringWithRange:selectedRange];
	NSString* bufString = [KStringUtils wrapString:target width:60];
	[self insertText:bufString];
	[self setSelectedRange:NSMakeRange(selectedRange.location, [bufString length])];
}

- (IBAction)removeReturns:(id)sender
{
	//NSLog(@"removeReturns:");
	NSRange selectedRange = [self selectedRange];
	NSString* string = [[self string] substringWithRange:selectedRange];
	string = [KStringUtils replaceString:string search:@"\r" replace:@""];
	string = [KStringUtils replaceString:string search:@"\n" replace:@""];
	[self insertText:string];
	[self setSelectedRange:NSMakeRange(selectedRange.location, [string length])];
}


- (IBAction)shiftRight:(id)sender
{
	if(![[self string] length]){ return; }
	
	NSRange selectedRange = [[self string] lineRangeForRange:[self selectedRange]];
	if([[[self string] substringWithRange:NSMakeRange(selectedRange.location + selectedRange.length - 1, 1)] isEqualToString:@"\r"]){
		selectedRange.length--;
	}
	//NSArray* lineArray = [[[self string] substringWithRange:selectedRange] componentsSeparatedByString:@"\r"];
	NSArray* lineArray = [KStringUtils string:[[self string] substringWithRange:selectedRange] componentsSeparatedByCharacters:@"\r\n"];
	NSMutableArray* newArray = [NSMutableArray arrayWithCapacity:[lineArray count]];
	unsigned i;
	
	for(i = 0; i < [lineArray count]; i++){
		//[newArray addObject:[@"\t" stringByAppendingString:[lineArray objectAtIndex:i]]];
		[newArray addObject:[NSString stringWithFormat:@"\t%@", [lineArray objectAtIndex:i]]];
		NSLog([lineArray objectAtIndex:i]);
	}
	NSString* newString = [newArray componentsJoinedByString:@"\r"];
	[self setSelectedRange:selectedRange];
	[self insertText:newString];
	[self setSelectedRange:NSMakeRange(selectedRange.location, [newString length])];
}

- (IBAction)shiftLeft:(id)sender
{
	if(![[self string] length]){ return; }
	
	NSRange selectedRange = [[self string] lineRangeForRange:[self selectedRange]];
	if([[[self string] substringWithRange:NSMakeRange(selectedRange.location + selectedRange.length - 1, 1)] isEqualToString:@"\r"]){
		selectedRange.length--;
	}
	NSArray* lineArray = [[[self string] substringWithRange:selectedRange] componentsSeparatedByString:@"\r"];
	NSMutableArray* newArray = [NSMutableArray arrayWithCapacity:[lineArray count]];
	unsigned i;
	NSString* string;
	
	for(i = 0; i < [lineArray count]; i++){
		string = [lineArray objectAtIndex:i];
		if(([string length] > 0) && [[string substringWithRange:NSMakeRange(0, 1)] isEqualToString:@"\t"]){
			[newArray addObject:[string substringWithRange:NSMakeRange(1, [string length] - 1)]];
		}else{
			[newArray addObject:string];
		}
	}
	NSString* newString = [newArray componentsJoinedByString:@"\r"];
	[self setSelectedRange:selectedRange];
	[self insertText:newString];
	[self setSelectedRange:NSMakeRange(selectedRange.location, [newString length])];
}

- (IBAction)saveDocument:(id)sender { [myDocument saveDocument:sender]; }
//- (IBAction)undo:(id)sender { [myDocument undo:sender]; }
- (BOOL)isAutoIndented { return [myDocument isAutoIndented]; }





 

@end






