//
//  MyDocument.h
//  KEdit
//
//  Created by Karino, Masatsugu on Tue Feb 18 2003.
//  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
//


#import <Cocoa/Cocoa.h>
#import "KTextUtils.h"
#import "KSearchServer.h"

#define KStoreWindowStatusForAttr @"kedit_store_window_status"
#define KTextSelectedRangeForAttr @"kedit_text_selected_range"

// constants
//float kUnwrappedViewWidth = 5000;

#define KPlainName @"Plain"
#define KHTMLName @"X/HTML"
#define KIndexName @"Indexed"
#define KKEditMarkupName @"k-markup"
#define KCSSName @"CSS"
#define KShellScriptName @"sh-script"
#define KPerlScriptName @"perl"
#define KRubyScriptName @"ruby"
#define KPHPName @"PHP"

typedef enum{
	KTextPlain = 1,
	KTextHTML,
	KTextIndex,
	KTextKEditMarkup,
	KTextCSS,
	KTextPerlScript,
	KTextPHP,
	KTextRubyScript,
	
	KTextShellScript
} KTextType;

@class KColorSyntaxServer;


@interface MyDocument : NSDocument
{
	@private
	IBOutlet id myTextView;
	IBOutlet id myCharacterCodePopupButton;
	IBOutlet id myReturnCodePopupButton;
	IBOutlet id myTextTypePopupButton;
	IBOutlet id myLineNumberView;
	IBOutlet id myScrollView;
	IBOutlet id myWindow;
	
	KColorSyntaxServer* mColorSyntaxServer;
	
	NSString* mBufString;
	NSRange mSelectedRange;
	
	NSStringEncoding 	mDocumentEncoding;
	KReturnCodeType 	mReturnCodeType;
	
	KTextType mTextType;
	
	NSString* mTextViewFontName;
	float mTextViewFontSize;
	
	BOOL mWordWrap;
	BOOL mAutoIndent;
	BOOL mStoreWindowStatus;
	
	
}

// from SearchServer
- (void)doSearch:(id)sender direction:(KSearchDirection)direction;
- (void)doReplaceAll:(id)sender;

//Hook Functions
//NSDocument
- (BOOL)revertToSavedFromFile:(NSString*)fileName ofType:(NSString*)type;
- (BOOL)validateMenuItem:(NSMenuItem*)anItem;
- (NSDictionary*)fileAttributesToWriteToFile:(NSString*)fullDocumentPath ofType:(NSString*)docType saveOperation:(NSSaveOperationType)saveOperationType;
	//- (BOOL)writeToFile:(NSString *)fileName ofType:(NSString *)docType;
//NSTextView
- (BOOL)textView:(NSTextView*)aTextView shouldChangeTextInRange:(NSRange)affectedCharRange replacementString:(NSString *)replacementString;
- (void)textViewDidChangeSelection:(NSNotification *)aNotification;
//NSSText
- (void)textDidChange:(NSNotification*)aNotification;
//NSWindow
- (void)windowDidBecomeMain:(NSNotification *)aNotification;

//access data-members
- (BOOL)isWordWrapped;
- (void)setWordWrapped:(BOOL)wordwrap;
- (BOOL)isAutoIndented;
- (void)setAutoIndented:(BOOL)autoindent;
- (NSString*)textViewFontName;
- (void)setTextViewFontName:(NSString*)fontName;
- (float)textViewFontSize;
- (void)setTextViewFontSize:(float)fontSize;
- (KTextType)textType;
- (void)setTextType:(KTextType)type;

- (NSStringEncoding)documentEncoding;
- (KReturnCodeType)returnCodeType;
- (NSFont*)textViewFont;


//internal use
- (void)setTextTypeWithTypeName:(NSString*)aType;// such as 'text file'; not KTextType.

- (void)showReplacedAllDialog:(unsigned)count;
- (void)showReplacedAllDialogEnded:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void*)contextInfo;

- (void)adjustPopupButtons;

- (void)validateMenus;
- (void)validateFontFamilyMenu;
- (void)validateFontSizeMenu;
- (void)validateTabWidthMenu;

- (void)attachTextTypeMenu;

- (void)adjustDocumentSettings;

// load setting file. at ~/Library/KEdit/settings/'name.type'
// if the file doesn't exist, load resfile from bundle.
- (NSDictionary*)loadSettingFileOfName:(NSString*)name ofType:(NSString*)type;

@end


@interface MyDocument (Actions)
//Actions for POPUPMENUs
- (IBAction)setCharacterCodeType:(id)sender;
- (IBAction)setReturnCodeType:(id)sender;
- (IBAction)textTypeMenuSelected:(id)sender;
//(use "charactercodetype" menu for force converting)
- (IBAction)forceToChangeEncodingDialogEnded:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void*)contextInfo;

//Actions for MENUs
- (IBAction)scrollToSelection:(id)sender;
- (IBAction)toggleWordWrap:(id)sender;
- (IBAction)toggleAutoIndent:(id)sender;
- (IBAction)toggleStoreWindowStatus:(id)sender;
- (IBAction)setFontFamily:(id)sender;
- (IBAction)setFontSize:(id)sender;

- (IBAction)showGotoLineWindow:(id)sender;

- (IBAction)showPreferenceWindow:(id)sender;
- (IBAction)showColorPanel:(id)sender;

- (IBAction)redrawColors:(id)sender;
- (IBAction)openWithBrowser:(id)sender;
- (IBAction)openUserDefinedTagFile:(id)sender;

- (IBAction)executeScript:(id)sender;

- (IBAction)showInformationDialog:(id)sender;
- (IBAction)showInformationDialogEnded:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void*)contextInfo;

//Actions for ColorPanel
- (IBAction)insertColorString:(id)sender;

//Actions for Search Functions.
- (IBAction)searchShowWindow:(id)sender;
- (IBAction)searchForward:(id)sender;
- (IBAction)searchBackward:(id)sender;
- (IBAction)setSelectionToSearch:(id)sender;
- (IBAction)replace:(id)sender;
- (IBAction)replaceAndFindAgain:(id)sender;

//Actions for BottomView.
- (IBAction)setSelectedRangeOfTextView:(id)sender;

//Actions for GotoLineWindow.
- (IBAction)setCurrentLineOfTextView:(id)sender;

@end


@interface MyDocument (ColorSyntax)

- (void)validateColorSyntax;
- (void)validateColorSyntax:(NSRange)validatedRange;

@end


@interface MyDocument (ScriptingSupport)

// Accessors.
- (NSString*)encoding;
- (void)setEncoding:(id)newEncoding;

- (NSString*)retcode;
- (void)setRetcode:(NSString*)newRetcode;

- (NSString*)syntaxtype;
- (void)setSyntaxtype:(NSString*)newSyntaxType;

- (NSString*)fontname;
- (void)setFontname:(NSString*)newFont;

- (NSNumber*)fontsize;
- (void)setFontsize:(NSNumber*)newSize;

- (NSDictionary*)selectedrange;
- (void)setSelectedrange:(NSDictionary*)newSelectedRange;

- (NSNumber*)wordwrap;
- (void)setWordwrap:(NSNumber*)newWordWrap;

- (NSNumber*)autoindent;
- (void)setAutoindent:(NSNumber*)newAutoIndent;

- (NSTextStorage*)selectedtext;
- (void)setSelectedtext:(NSTextStorage*)newText;

- (NSNumber*)selectionLocation;
- (void)setSelectionLocation:(NSNumber*)newSelectionLocation;
- (NSNumber*)selectionLength;
- (void)setSelectionLength:(NSNumber*)newSelectionLength;

- (NSTextStorage*)text;

// ScriptCommands.
- (NSNumber*)handleReplaceAllCommand:(NSScriptCommand*)command;
- (NSNumber*)handleSearchCommand:(NSScriptCommand*)command;

- (void)handleScrollToSelectionCommand:(NSScriptCommand*)command;

@end








