//
//  MyDocument.m
//  KEdit
//
//  Created by Karino, Masatsugu on Tue Feb 18 2003.
//  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
//
#import <Carbon/Carbon.h>

#import "MyDocument.h"
#import "KTextUtils.h"
#import "KApplicationDel.h"
#import "KPrefsServer.h"
#import "KSearchServer.h"
#import "KMenuUtils.h"
#import "KStringUtils.h"
#import "KTextView.h"
#import "KRangeObj.h"
#import "KColorSyntaxServer.h"
#import "KSettingFile.h"
#import "KAddon.h"

@implementation MyDocument

- (id)init
{
    [super init];
    if (self) {
        // Add your subclass-specific initialization here.
        // If an error occurs here, send a [self dealloc] message and return nil.
		
		//set initial data to data-members.
		KPrefsServer* prefs = [[NSApp delegate] prefsServer];
		
		mBufString = nil;
		mReturnCodeType = KCR_type; // = mUserDefinedReturnCodeType
		mDocumentEncoding = NSShiftJISStringEncoding;
		mTextType = KTextPlain;
		mSelectedRange = NSMakeRange(0, 0);
		
		mWordWrap = [prefs boolForKey:KWordWrap];
		mAutoIndent = [prefs boolForKey:KAutoIndent];
		mStoreWindowStatus = [prefs boolForKey:KStoreWindowStatus];
		mTextViewFontName = [[NSString alloc] initWithString:[prefs objectForKey:KFontName]];
		mTextViewFontSize = [prefs floatForKey:KFontSize];
		
		mColorSyntaxServer = nil;
    }
    return self;
}

- (void)dealloc
{
	[mColorSyntaxServer release];
	[mTextViewFontName release];
	
	[super dealloc];
}

- (id)initWithContentsOfFile:(NSString*)fileName ofType:(NSString*)docType
{
	[super initWithContentsOfFile:fileName ofType:docType];
	return self;
}


- (NSString *)windowNibName
{
    // Override returning the nib file name of the document
    // If you need to use a subclass of NSWindowController or if your document supports multiple NSWindowControllers, you should remove this method and override -makeWindowControllers instead.
    return @"MyDocument";
}

- (NSData *)dataRepresentationOfType:(NSString *)aType
{
    // Insert code here to write your document from the given data.  You can also choose to override -fileWrapperRepresentationOfType: or -writeToFile:ofType: instead.
    
	NSString* string = [myTextView string];
	string = [KStringUtils convertReturnsFromCR:string to:mReturnCodeType];
	NSData* aData = [string dataUsingEncoding:[self documentEncoding]];
	//aData = [KTextUtils convertReturnCode:aData from:mReturnCodeType to:mUserDefinedReturnCodeType];
	
	//[[NSWorkspace sharedWorkspace] setFinderInfo:[self fileName] fileType:@"TEXT" creator:@"KEDI"];
	
	return aData;
	
}

- (BOOL)loadDataRepresentation:(NSData *)data ofType:(NSString *)aType
{
    // Insert code here to read your document from the given data.  You can also choose to override -loadFileWrapperRepresentation:ofType: or -readFromFile:ofType: instead.
    
	//set data-members.
	//mReturnCodeType = mUserDefinedReturnCodeType = [KTextUtils detectReturnCodeType:data];
	if([data length]){
		mDocumentEncoding = [KTextUtils detectCharacterCodeType:data];
	}
	
	NSString* buf = [[NSString alloc] initWithData:data encoding:mDocumentEncoding];
	if(!buf){
		// default encoding == S-JIS
		mDocumentEncoding = NSShiftJISStringEncoding;
		buf = [[NSString alloc] initWithData:data encoding:mDocumentEncoding];
	}
	
	mBufString = [[KStringUtils convertReturnsToCR:buf from:&mReturnCodeType] retain];
	[buf release];

	
	[self setTextTypeWithTypeName:aType];
	
	return YES;
}

// The last thing to do for the Window.
- (void)windowControllerDidLoadNib:(NSWindowController*)windowController
{
	[super windowControllerDidLoadNib:windowController];
	[[myTextView window] makeFirstResponder:myTextView];
	
	if(mBufString == nil){ // new document.
		[myTextView setString:@""];
	}else{ // open document.
		NSRange range;
		NSDictionary* dict = [[NSFileManager defaultManager] fileAttributesAtPath:[self fileName] traverseLink:YES];
		//NSLog([NSString stringWithFormat:@"count = %d", [dict count]]);
		//mStoreWindowStatus = [[dict objectForKey:KStoreWindowStatusForAttr] boolValue];
		if(dict && mStoreWindowStatus){
			range = NSRangeFromString([dict objectForKey:KTextSelectedRangeForAttr]);
		}else{
			range = NSMakeRange(0, 0);
		}
		[myTextView setString:mBufString];
		[myTextView setSelectedRange:range];
		
		[mBufString release];
		mBufString = nil;
	}
	
	[self setWordWrapped:mWordWrap];
	
	[self adjustDocumentSettings];
	
	[self validateColorSyntax];
	[self attachTextTypeMenu];
	[self adjustPopupButtons];
	[myLineNumberView setStringValue:@"1"];
	
	// SET TEXTVIEW FRAME.
	NSRect textviewFrame = [myTextView frame];
	textviewFrame.origin = NSMakePoint(textviewFrame.origin.x - 1, textviewFrame.origin.y - 1);
	textviewFrame.size = NSMakeSize(textviewFrame.size.width + 2, textviewFrame.size.height + 2);	
	[myTextView setFrame:textviewFrame];
	
	// SET WINDOW FRAME. 
	NSRect windowFrame;
	NSRect screenFrame = [[NSScreen mainScreen] frame];
	//NSLog([NSString stringWithFormat:@"height = %f", screenFrame.size.height]);
	windowFrame.size = NSMakeSize(600.0, 600.0);
	windowFrame.origin = NSMakePoint(30.0, screenFrame.size.height - windowFrame.size.height - 30.0);
	[myWindow setFrame:windowFrame display:YES];
	
	//[myTextView registerForDraggedTypes:[NSArray arrayWithObjects:NSColorPboardType, NSFilenamesPboardType, nil]]; 
	//[[myTextView window] registerForDraggedTypes:[NSArray arrayWithObjects:NSColorPboardType, NSFilenamesPboardType, nil]]; 

}


////////
// from SearchServer
////////

- (void)doSearch:(id)sender direction:(KSearchDirection)direction
{
	id searchServer = [[NSApp delegate] searchServer];
	NSRange selectedRange = [searchServer searchString:myTextView direction:direction];
	if(selectedRange.location == NSNotFound){
		NSBeep();
	}else{
		[myTextView setSelectedRange:selectedRange];
		[myTextView scrollRangeToVisible:selectedRange];
	}
}

- (void)doReplaceAll:(id)sender
{
	id searchServer = [[NSApp delegate] searchServer];
	unsigned count = [searchServer replaceAll:myTextView];
	[self validateColorSyntax];
	[myTextView setSelectedRange:NSMakeRange(0, 0)];
	
	[self showReplacedAllDialog:count];
}


- (NSDictionary*)fileAttributesToWriteToFile:(NSString*)fullDocumentPath ofType:(NSString*)docType saveOperation:(NSSaveOperationType)saveOperationType
{
	NSNumber* creatorCode = nil;
	NSNumber* typeCode = nil;
	KPrefsServer* prefs = [[NSApp delegate] prefsServer];
	NSDictionary* infoPlist = [[NSBundle mainBundle] infoDictionary];
	NSMutableDictionary* newList = [NSMutableDictionary dictionaryWithDictionary:[super fileAttributesToWriteToFile:fullDocumentPath ofType:docType saveOperation:saveOperationType]];
	NSString* creatorCodeString = [infoPlist objectForKey:@"CFBundleSignature"];
	
	
	//[newList setObject:@"kedit-sample" forKey:@"sample"];
	/*
	if(mStoreWindowStatus){
		NSRange selectedRange = [myTextView selectedRange];
		[newList setObject:NSStringFromRange(selectedRange) forKey:KTextSelectedRangeForAttr];
		//NSRect textviewBounds = [myTextView bounds];
	}
	[newList setBool:mStoreWindowStatus forKey:KStoreWindowStatusForAttr];*/
	
	if(creatorCodeString && [prefs boolForKey:KStoreTextType]){
		creatorCode = [NSNumber numberWithUnsignedLong:NSHFSTypeCodeFromFileType([NSString stringWithFormat:@"'%@'", creatorCodeString])];
		[newList setObject:creatorCode forKey:NSFileHFSCreatorCode];
		
		typeCode = [NSNumber numberWithUnsignedLong:NSHFSTypeCodeFromFileType(@"'TEXT'")];
		[newList setObject:typeCode forKey:NSFileHFSTypeCode];
	}
	return newList;
}



- (void)showReplacedAllDialog:(unsigned)count
{
	NSString* alertString = [NSString stringWithFormat:
					@"Replaced %d words",
					count];

	NSBeginAlertSheet(
            @"Replacement Information", 
            @"OK", 
            nil, 
            nil, 
            myWindow, 
            self, 
			@selector(showReplacedAllDialogEnded:returnCode:contextInfo:),
            nil, 
            nil, 
            alertString);
}

- (void)showReplacedAllDialogEnded:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void*)contextInfo
{
	//Do nothing...
}




////////
// Hook Functions
////////

- (BOOL)revertToSavedFromFile:(NSString*)fileName ofType:(NSString*)type
{
	NSData* data = [NSData dataWithContentsOfFile:fileName];
	
	if(!data && [data length]){
		mDocumentEncoding = [KTextUtils detectCharacterCodeType:data];
	}
	
	NSString* buf = [[NSString alloc] initWithData:data encoding:mDocumentEncoding];
	if(!buf){
		// default encoding == S-JIS
		mDocumentEncoding = NSShiftJISStringEncoding;
		buf = [[NSString alloc] initWithData:data encoding:mDocumentEncoding];
		if(!buf){ return NO; }
	}
	
	NSString* convertedString = [[KStringUtils convertReturnsToCR:buf from:&mReturnCodeType] retain];
	[buf release];
	[myTextView setString:convertedString];
	[myTextView setSelectedRange:NSMakeRange(0, 0)];
	[self adjustPopupButtons];
	[self setTextTypeWithTypeName:type];
	[self validateColorSyntax];
	
	return YES;
}

- (BOOL)validateMenuItem:(NSMenuItem*)anItem
{
	[super validateMenuItem:anItem];
	
	NSString* itemTitle = [anItem title];
	NSString* menuTitle = [[anItem menu] title];
	
	if([itemTitle isEqualToString:NSLocalizedString(@"Word Wrap",@"")]){
		[anItem setState:mWordWrap];
		
	}else if([itemTitle isEqualToString:NSLocalizedString(@"Auto Indent",@"")]){
		[anItem setState:mAutoIndent];
		
	}else if([itemTitle isEqualToString:NSLocalizedString(@"Store Window Status",@"")]){
		[anItem setState:mStoreWindowStatus];
	
	}else if([itemTitle isEqualToString:NSLocalizedString(@"Open User-defined Tag File",@"")]){
		NSFileManager* manager = [NSFileManager defaultManager];
		NSString* dirName = [NSHomeDirectory() stringByAppendingPathComponent:KPreferenceDirectory];
		NSString* tagFileName = [dirName stringByAppendingPathComponent:KTagFileName];
		return [manager fileExistsAtPath:tagFileName];
		
	}else if([menuTitle isEqualToString:NSLocalizedString(@"Save", @"")]){
		if(![self isDocumentEdited]){ return NO; }
	}
	return YES;
}


// NSTextView's delegate method.
- (BOOL)textView:(NSTextView*)aTextView shouldChangeTextInRange:(NSRange)affectedCharRange replacementString:(NSString *)replacementString
{
	//When deleting, get empty string @"".
	mSelectedRange = NSMakeRange(affectedCharRange.location, [replacementString length]);
	return YES;
}

/*
- (BOOL)textView:(NSTextView*)aTextView doCommandBySelector:(SEL)selector
{
	//NSLog(@"doCommand");
	//return YES, do nothing by textview, return NO, do command by textveiw.
	//Events like "tab key", "delete key" and so on.
	return NO;
}*/

- (void)textViewDidChangeSelection:(NSNotification *)aNotification
{
	/*
	NSRange selectedRange = [myTextView selectedRange];
	NSString* string = [myTextView string];
	NSRange currentRange = NSMakeRange(0, 0);
	NSRange bufRange;
	unsigned lineNum = 0;
	
	while(YES){
		//NSLog([NSString stringWithFormat:@"range = (%d, %d)", currentRange.location, currentRange.length]);
		bufRange = [string lineRangeForRange:NSMakeRange(NSMaxRange(currentRange), 0)];
		if(NSEqualRanges(bufRange, currentRange)){
			if(lineNum != 0){ lineNum--; }
			break;
		}
		if(NSLocationInRange(selectedRange.location, bufRange)){ break; }
		currentRange = bufRange;
		lineNum++;
	}
	[myLineNumberView setStringValue:[NSString stringWithFormat:@"%d", lineNum + 1]];*/
	
	NSRange selectedRange = [myTextView selectedRange];
	unsigned lineNum = [KStringUtils countLinesOfString:[myTextView string] range:NSMakeRange(0, selectedRange.location)];
	[myLineNumberView setStringValue:[NSString stringWithFormat:@"%d", lineNum]];
}

// NSText's delegate method.
- (void)textDidChange:(NSNotification*)aNotification
{
	//This method will be called after textView:shouldChangetextInRange:replacementString:.
	//let document instance know the document had been edited.
	[self validateColorSyntax:mSelectedRange];
	[self updateChangeCount:NSChangeDone];
}

// NSWindow's delegate method.
- (void)windowDidBecomeMain:(NSNotification *)aNotification
{
	//[[NSApp delegate] constructMenus];
	[self validateMenus];
}



////////
// internal use
////////
- (void)setTextTypeWithTypeName:(NSString*)aType
{
	if([aType isEqualToString:@"text file"]){
		mTextType = KTextPlain;
	}else if([aType isEqualToString:@"HTML file"]){
		mTextType = KTextHTML;
	}else if([aType isEqualToString:@"css file"]){
		mTextType = KTextCSS;
	}else if([aType isEqualToString:@"php file"]){
		mTextType = KTextPHP;
	}else if([aType isEqualToString:@"perl file"]){
		mTextType = KTextPerlScript;
	}else if([aType isEqualToString:@"ruby file"]){
		mTextType = KTextRubyScript;
	}
}

- (void)adjustPopupButtons
{
	// Tag: 0=SJIS, 1=JIS, 2=EUC, 3=UTF-8, 4=UTF-16
	int popupIndex;
	switch(mDocumentEncoding){
		case NSShiftJISStringEncoding: popupIndex = 0; break;
		case NSISO2022JPStringEncoding: popupIndex = 1; break;
		case NSJapaneseEUCStringEncoding: popupIndex = 2; break;
		case NSUTF8StringEncoding: popupIndex = 3; break;
		case NSUnicodeStringEncoding : popupIndex = 4; break;
	}
	[myCharacterCodePopupButton selectItemAtIndex:popupIndex];
	// viewWithTag: method must be used.
	
	//Tag: 0=LF, 1=CR, 2=CR-LF
	switch(mReturnCodeType){
		case KLF_type: popupIndex = 0; break;
		case KCR_type: popupIndex = 1; break;
		case KCRLF_type: popupIndex = 2; break;
	}
	[myReturnCodePopupButton selectItemAtIndex:popupIndex];
	
	////////
	// TextTypePopupMenu
	////////
	NSMenuItem* menuItem = [KMenuUtils searchMenuItemWithTag:mTextType targetMenu:[myTextTypePopupButton menu]];
	[myTextTypePopupButton selectItem:menuItem];
}


- (void)validateMenus
{
	[self validateFontFamilyMenu];
	[self validateFontSizeMenu];
	[self validateTabWidthMenu];
}

- (void)validateFontFamilyMenu
{
	NSMenuItem* menuItem = [KMenuUtils searchMenuItemOfMenu:[NSApp mainMenu] localizedString:KFontFamilyMenuItemName];
	[KMenuUtils turnOffAllMenuItemState:[menuItem submenu]];
	
	NSArray* fontArray = [[NSApp delegate] menuDict:KFontFamilyArray];
	unsigned tag = [fontArray indexOfObject:mTextViewFontName];
	if(tag != NSNotFound){
		NSMenuItem* fontMenuItem = [KMenuUtils searchMenuItemWithTag:tag targetMenu:[menuItem submenu]];
		if(fontMenuItem){
			[fontMenuItem setState:NSOnState];
			
			NSMenuItem* upperMenuItem = [KMenuUtils getMenuItemWithSubmenu:[fontMenuItem menu]];
			if(upperMenuItem){ [upperMenuItem setState:NSOnState]; }
		}else{ NSLog(@"fontMenuItem : NotFound"); }
	}else{ NSLog(@"TextViewFontName : NotFound"); }
}

- (void)validateFontSizeMenu
{
	NSMenuItem* menuItem = [KMenuUtils searchMenuItemOfMenu:[NSApp mainMenu] localizedString:KFontSizeMenuItemName];
	[KMenuUtils turnOffAllMenuItemState:[menuItem submenu]];
	
	NSArray* sizeArray = [[NSApp delegate] menuDict:KFontSizeArray];
	unsigned tag = [sizeArray indexOfObject:[NSNumber numberWithFloat:mTextViewFontSize]];
	if(tag != NSNotFound){
		NSMenuItem* sizeMenuItem = [KMenuUtils searchMenuItemWithTag:tag targetMenu:[menuItem submenu]];
		if(sizeMenuItem){
			[sizeMenuItem setState:NSOnState];
		}
	}
}

- (void)validateTabWidthMenu
{
	
}

- (void)attachTextTypeMenu
{
	NSMenu* menu = [[NSMenu alloc] initWithTitle:@""];
	
	NSMenuItem* menuItem = [[NSMenuItem alloc] initWithTitle:KPlainName action:@selector(textTypeMenuSelected:) keyEquivalent:@"/"];
	[menuItem setTag:KTextPlain];
	[menu addItem:menuItem];
	
	menuItem = [[NSMenuItem alloc] initWithTitle:KHTMLName action:@selector(textTypeMenuSelected:) keyEquivalent:@"m"];
	[menuItem setTag:KTextHTML];
	[menu addItem:menuItem];
	
	menuItem = [[NSMenuItem alloc] initWithTitle:KCSSName action:@selector(textTypeMenuSelected:) keyEquivalent:@""];
	[menuItem setTag:KTextCSS];
	[menu addItem:menuItem];
	
	menuItem = [[NSMenuItem alloc] initWithTitle:KPHPName action:@selector(textTypeMenuSelected:) keyEquivalent:@""];
	[menuItem setTag:KTextPHP];
	[menu addItem:menuItem];
	
	menuItem = [[NSMenuItem alloc] initWithTitle:KRubyScriptName action:@selector(textTypeMenuSelected:) keyEquivalent:@""];
	[menuItem setTag:KTextRubyScript];
	[menu addItem:menuItem];
	
	menuItem = [[NSMenuItem alloc] initWithTitle:KPerlScriptName action:@selector(textTypeMenuSelected:) keyEquivalent:@""];
	[menuItem setTag:KTextPerlScript];
	[menu addItem:menuItem];
	
	[menu addItem:[NSMenuItem separatorItem]];
	
	menuItem = [[NSMenuItem alloc] initWithTitle:KIndexName action:@selector(textTypeMenuSelected:) keyEquivalent:@""];
	[menuItem setTag:KTextIndex];
	[menu addItem:menuItem];
	
	menuItem = [[NSMenuItem alloc] initWithTitle:KKEditMarkupName action:@selector(textTypeMenuSelected:) keyEquivalent:@""];
	[menuItem setTag:KTextKEditMarkup];
	[menu addItem:menuItem];
	
	[myTextTypePopupButton setMenu:menu];
	
}

- (void)adjustDocumentSettings
{
	// set ColorSyntaxServer
	[mColorSyntaxServer release];
	mColorSyntaxServer = [[KColorSyntaxServer alloc] initWithTextStorage:[myTextView textStorage] ofType:mTextType];
	
	NSTextStorage* storage = [myTextView textStorage];
	NSRange validatedRange = NSMakeRange(0, [storage length]);
	
	// set Font size
	if([[mColorSyntaxServer settings] settingForKey:KModeSettingsDefault_FontSize]){ // dirty...
		float fontSize = [[mColorSyntaxServer settings] fontSize];
		if(fontSize){
			[self setTextViewFontSize:fontSize];
			NSFont* font = [NSFont fontWithName:mTextViewFontName size:fontSize];
			[storage addAttribute:NSFontAttributeName value:font range:validatedRange];
		}
	}
	
	// set word-wrap
	if([[mColorSyntaxServer settings] settingForKey:KModeSettingsDefault_IsWordWrapped]){ // dirty...
		BOOL isWordWrapped = [[mColorSyntaxServer settings] isWordWrapped];
		[self setWordWrapped:isWordWrapped];
	}
}

- (NSDictionary*)loadSettingFileOfName:(NSString*)name ofType:(NSString*)type
{
	NSFileManager* manager = [NSFileManager defaultManager];
	NSBundle* bundle = [NSBundle mainBundle];
	NSString* bundleSettingName = [bundle pathForResource:name ofType:type];
	if(!bundleSettingName){ return nil; }
	NSMutableDictionary* defDict = [NSMutableDictionary dictionaryWithDictionary:[KStringUtils settingsFromString:[NSString stringWithContentsOfFile:bundleSettingName]]];
	NSDictionary* userDict;
	
	NSString* settingFileName = [NSString stringWithFormat:@"%@/%@/%@/%@.%@", 
									NSHomeDirectory(), KPreferenceDirectory, KSettingsDirectoryName, name, type];
	//NSLog(settingFileName);
	if([manager fileExistsAtPath:settingFileName]){
		userDict = [KStringUtils settingsFromString:[NSString stringWithContentsOfFile:settingFileName]];
		[defDict addEntriesFromDictionary:userDict];
	}
	return defDict;
}


// for accessing data-members.
- (void)setWordWrapped:(BOOL)wordwrap
{
	NSRect frame = [myTextView frame];
	
	if(!wordwrap){
		frame.size.width = 5000;//kUnwrappedViewWidth;
		[myTextView setFrame:frame];
		[myScrollView setHasHorizontalScroller:YES];
		
	}else{
		NSRect scrollFrame = [[myScrollView horizontalScroller] frame];
		frame.size.width = scrollFrame.size.width;
		[myTextView setFrame:frame];
		[myScrollView setHasHorizontalScroller:NO];
	}
	mWordWrap = wordwrap;
}

- (void)setTextViewFontName:(NSString*)fontName
{
	[mTextViewFontName release];
	mTextViewFontName = [[NSString alloc] initWithString:fontName];
	[self validateFontFamilyMenu];
}

- (void)setTextViewFontSize:(float)fontSize
{
	mTextViewFontSize = fontSize;
	[self validateFontSizeMenu];
}

- (void)setTextType:(KTextType)type
{
	mTextType = type;
	[self adjustPopupButtons];
}


- (NSStringEncoding)documentEncoding { return mDocumentEncoding; }
- (KReturnCodeType)returnCodeType { return mReturnCodeType; }
- (KTextType)textType { return mTextType; }
- (BOOL)isWordWrapped { return mWordWrap; }
- (BOOL)isAutoIndented { return mAutoIndent; }
- (void)setAutoIndented:(BOOL)autoindent { mAutoIndent = autoindent; }
- (NSFont*)textViewFont { return [NSFont fontWithName:mTextViewFontName size:mTextViewFontSize]; }
- (NSString*)textViewFontName { return mTextViewFontName; }
- (float)textViewFontSize { return mTextViewFontSize; }


@end



@implementation MyDocument (Actions)
////////
// Actions.
////////
- (IBAction)setCharacterCodeType:(id)sender
{
	// Tag: 0=SJIS, 1=JIS, 2=EUC, 3=UTF-8, 4=UTF-16
	//NSLog(@"setCharacterType:");
	
	int aTag = [[sender selectedItem] tag];
	
	switch(aTag){
		case 0: mDocumentEncoding = NSShiftJISStringEncoding; break;
		case 1: mDocumentEncoding = NSISO2022JPStringEncoding; break;
		case 2: mDocumentEncoding = NSJapaneseEUCStringEncoding; break;
		case 3: mDocumentEncoding = NSUTF8StringEncoding; break;
		case 4: mDocumentEncoding = NSUnicodeStringEncoding; break;
	}
	
	if(([self fileName] != nil) && (GetCurrentKeyModifiers() & shiftKey)){
		NSString* alertString = 
			@"Reload this document and force to convert into the selected character code.\n"
			@"Your changes will be lost.";
	
		NSBeginAlertSheet(
            @"Alert", 
            @"OK", 
            @"Cancel", 
            nil, 
            myWindow, 
            self, 
			@selector(forceToChangeEncodingDialogEnded:returnCode:contextInfo:),
            nil, 
            nil, 
            alertString);
	}
}

- (IBAction)setReturnCodeType:(id)sender
{
	//Tag: 0=LF, 1=CR, 2=CR-LF
	//NSLog(@"setReturnCodeType:");
	
	int aTag = [[sender selectedItem] tag];
	
	switch(aTag){
		case 0: mReturnCodeType = KLF_type; break;
		case 1: mReturnCodeType = KCR_type; break;
		case 2: mReturnCodeType = KCRLF_type; break;
	}
}

- (IBAction)scrollToSelection:(id)sender
{
	//NSLog(@"scrollToSelection");
	[myTextView scrollRangeToVisible:[myTextView selectedRange]];
}


- (IBAction)toggleWordWrap:(id)sender
{
	[self setWordWrapped:!mWordWrap];
}

- (IBAction)toggleAutoIndent:(id)sender
{
	mAutoIndent = !mAutoIndent;
}

- (IBAction)toggleStoreWindowStatus:(id)sender
{
	mStoreWindowStatus = !mStoreWindowStatus;
}

- (IBAction)setFontFamily:(id)sender
{
	int tag = [sender tag];
	NSString* fontName = [[[NSApp delegate] menuDict:KFontFamilyArray] objectAtIndex:tag];

	[self setTextViewFontName:fontName];
	[self validateColorSyntax];
	
}

- (IBAction)setFontSize:(id)sender
{	
	int tag = [sender tag];
	[self setTextViewFontSize:KFontSizeMenuItemArray[tag]];
	[self validateColorSyntax];
}



- (IBAction)showPreferenceWindow:(id)sender
{
	[[[NSApp delegate] prefsServer] showPreferenceWindow];
}

- (IBAction)showColorPanel:(id)sender
{
	//NSLog(@"showColorPanel:");
	
	NSString* string = [[myTextView string] substringWithRange:[myTextView selectedRange]];
	NSColor* color = [KStringUtils colorFromString:string];
	
	NSColorPanel* panel = [NSColorPanel sharedColorPanel];
	[panel setTarget:self];
	[panel setAction:@selector(insertColorString:)];
	if(color){ [panel setColor:color]; }
	[panel makeKeyAndOrderFront:self];
}

- (IBAction)insertColorString:(id)sender
{
	//NSLog(@"MyDocument : insertColorString:");
	[myTextView insertColorString:[sender color]];
}

- (IBAction)redrawColors:(id)sender
{
	[self validateColorSyntax];
}

- (IBAction)openWithBrowser:(id)sender
{
	if(![self fileName]){ NSBeep(); return; }
	if([self isDocumentEdited]){ [self saveDocument:self]; }
	
	NSFileManager* manager = [NSFileManager defaultManager];
	NSArray* components = [manager componentsToDisplayForPath:[self fileName]];
	NSString* newPath = [components componentsJoinedByString:@":"];
	NSString* newURL = [components componentsJoinedByString:@"/"];
	KPrefsServer* prefs = [[NSApp delegate] prefsServer];
	NSString* scriptString;
	
	if([prefs intForKey:KPreferredBrowser] == KInternetExplorer){
		if([prefs boolForKey:KAlwaysMakesANewWindow]){ // loaed into the new window of IE.
			scriptString = [NSString stringWithFormat:
				@"tell application \"Internet Explorer\"\r"
				@"activate\r"
				@"open alias \"%@\"\r"
				@"end tell\r"
				, newPath];
			
		}else{ // loaed into the frontmost window of IE.
			scriptString = [NSString stringWithFormat:
				@"tell application \"Internet Explorer\"\r"
				@"activate\r"
				@"ShowFile alias \"%@\" URL \"file://localhost/Volumes/%@\"\r"
				@"end tell\r"
				, newPath, newURL];
		}
	
		NSAppleScript* appleScript = [[NSAppleScript alloc] initWithSource:scriptString];
		NSDictionary* dict;
		[appleScript executeAndReturnError:&dict];
		[appleScript release];
		
	}else if([prefs intForKey:KPreferredBrowser] == KSafari){
		/* Currently Safari can't be controlled with AppleScript enoughly. */
		/* The routines below don't act properly. */
		if([prefs boolForKey:KAlwaysMakesANewWindow]){
			scriptString = [NSString stringWithFormat:
				@"tell application \"Safari\"\r"
				@"activate\r"
				@"open alias \"%@\"\r"
				@"end tell\r"
				, newPath];
		}else{
			/*
			scriptString = [NSString stringWithFormat:
				@"tell application \"Safari\"\r"
				@"activate\r"
				@"set URL of document 1 to \"file:///Volumes/%@\"\r"
				@"end tell\r"
				, newURL];*/
			/* this code below was written by Mr. Hetima. Thanx! */
			NSURL* tempURL=[[[NSURL alloc]initFileURLWithPath:[self fileName]]autorelease];
			scriptString = [NSString stringWithFormat:
				@"tell application \"Safari\"\r"
				@"activate\r"
				@"reopen\r"
				@"set URL of document 1 to \"%@\"\r"
				@"end tell\r"
				, [tempURL absoluteString]];
		}
		NSAppleScript* appleScript = [[NSAppleScript alloc] initWithSource:scriptString];
		NSDictionary* dict;
		[appleScript executeAndReturnError:&dict];
		[appleScript release];
	}
}

- (IBAction)executeScript:(id)sender
{

	//NSLog([sender attachedObj]);
	
	NSString* path = [sender attachedObj];
	if(GetCurrentKeyModifiers() & shiftKey){
		NSWorkspace* workspace = [NSWorkspace sharedWorkspace];
		[workspace openFile:path withApplication:@"KEdit"];
	}else{
		KAddon* addon = [KAddon addonWithContentsOfFile:path];
		KAddonEditingType type;
		NSString* resString = [addon runAddonFor:[myTextView string] range:[myTextView selectedRange] type:&type path:[self fileName]];
		if(type == KReplaceSelection){
			[myTextView insertText:resString];
		}else if(type == KReplaceAll){
			[myTextView setString:resString];
		}else{
			// do nothing.
		}
		
	}
}

- (IBAction)openUserDefinedTagFile:(id)sender
{
	NSWorkspace* workspace = [NSWorkspace sharedWorkspace];
	NSString* dirName = [NSHomeDirectory() stringByAppendingPathComponent:KPreferenceDirectory];
	NSString* tagFileName = [dirName stringByAppendingPathComponent:KTagFileName];
	
	BOOL wasOpened = [workspace openFile:tagFileName];
	
	if(!wasOpened){
		NSLog(@"MyDocument.m : openUserDefinedTagFile: - no such file.");
	}
}

- (IBAction)textTypeMenuSelected:(id)sender
{
	[self setTextType:[sender tag]];
	[self adjustDocumentSettings];
	[self validateColorSyntax];
}

- (IBAction)showInformationDialog:(id)sender
{
	//APPKIT_EXTERN void NSBeginAlertSheet(
	//		NSString *title, NSString *defaultButton, NSString *alternateButton, NSString *otherButton,
	//		NSWindow *docWindow, id modalDelegate,
	//		SEL didEndSelector, SEL didDismissSelector,
	//		void *contextInfo, NSString *msg, ...);
	
	unsigned stringLength = [[myTextView string] length];
	unsigned lineCount = [KStringUtils countLinesOfString:[myTextView string]];
	NSRange selectedRange = [myTextView selectedRange];
	NSString* substring = [[myTextView string] substringWithRange:selectedRange];
	unsigned substringLength = [substring length];
	unsigned sublineCount = [KStringUtils countLinesOfString:substring];
	
	NSString* alertString = [NSString stringWithFormat:
					@"Whole Text :\n  Character Count : %d\n  Line Count : %d\n\n"
					@"Selected Text :\n  Character Count : %d\n  Line Count : %d",
					stringLength, lineCount,
					substringLength, sublineCount];

	NSBeginAlertSheet(
            @"Text Information", 
            @"OK", 
            nil, 
            nil, 
            myWindow, 
            self, 
			@selector(showInformationDialogEnded:returnCode:contextInfo:),
            nil, 
            nil, 
            alertString);
}

- (IBAction)showInformationDialogEnded:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void*)contextInfo
{
	//Do nothing...
	//NSLog(@"showInformationDialogEnded:::");
}



- (IBAction)forceToChangeEncodingDialogEnded:(NSWindow*)sheet returnCode:(int)returnCode contextInfo:(void*)contextInfo
{
	//NSLog([NSString stringWithFormat:@"retCode=%d", returnCode]);
	if(returnCode == 1){ // pressed 'OK' button.
		NSData* data = [NSData dataWithContentsOfFile:[self fileName]];
		NSString* bufString = [[NSString alloc] initWithData:data encoding:mDocumentEncoding];
		NSString* modString = [KStringUtils convertReturnsToCR:bufString from:nil];
		[myTextView setString:modString];
		[myTextView setSelectedRange:NSMakeRange(0, 0)];
		[bufString release];
	}
}

//Actions for Search Functions.
- (IBAction)searchShowWindow:(id)sender
{
	id searchServer = [[NSApp delegate] searchServer];
	[searchServer showSearchWindow:self];
}
- (IBAction)searchForward:(id)sender
{
	[self doSearch:self direction:KSearchForward];
}
- (IBAction)searchBackward:(id)sender
{
	[self doSearch:self direction:KSearchBackward];
}
- (IBAction)setSelectionToSearch:(id)sender
{
	NSString* currentString = [[myTextView textStorage] string];
	NSRange selectedRange = [myTextView selectedRange];
	[[[NSApp delegate] searchServer] setSearchString:[currentString substringWithRange:selectedRange]];
	
}

- (IBAction)replace:(id)sender
{
	KSearchServer* searchServer = [[NSApp delegate] searchServer];
	NSRange selectedRange = [myTextView selectedRange];
	if(selectedRange.length != 0){
		[myTextView insertText:[searchServer replaceString]];
		[myTextView setSelectedRange:NSMakeRange(selectedRange.location, [[searchServer replaceString] length])];
	}
}

- (IBAction)replaceAndFindAgain:(id)sender
{
	KSearchServer* searchServer = [[NSApp delegate] searchServer];
	if([myTextView selectedRange].length != 0){
		[myTextView insertText:[searchServer replaceString]];
	}
	[self doSearch:self direction:KSearchForward];
}

//Actions for BottomView
- (IBAction)setSelectedRangeOfTextView:(id)sender
{
	//[myTextView setSelectedRange:NSRangeFromString([sender attachedObj])];
	//[myTextView scrollRangeToVisible:NSRangeFromString([sender attachedObj])];
	NSRange range = [[sender attachedObj] range];
	[myTextView setSelectedRange:range];
	[myTextView scrollRangeToVisible:range];
}

//Actions for GotoLineWindow
- (IBAction)setCurrentLineOfTextView:(id)sender
{
	//NSLog(@"gotoLine");
	if(![[sender stringValue] isEqualToString:@""]){
		[myTextView selectLineOf:[[sender stringValue] intValue]];
	}
	[[NSApp delegate] hideGotoLineWindow];
}


- (IBAction)showGotoLineWindow:(id)sender
{
	[[NSApp delegate] showGotoLineWindow];
}


@end



@implementation MyDocument (ColorSyntax)

- (void)validateColorSyntax
{
	[self validateColorSyntax:NSMakeRange(0, [[myTextView string] length])];
}

- (void)validateColorSyntax:(NSRange)validatedRange
{
	[mColorSyntaxServer adjustColorSyntaxWithRange:validatedRange andFont:[self textViewFont]];
}

@end


@implementation MyDocument (ScriptingSupport)

- (NSString*)encoding
{
	switch((int)mDocumentEncoding){
		case NSShiftJISStringEncoding: return @"S-JIS";
		case NSJapaneseEUCStringEncoding: return @"EUC";
		case NSUTF8StringEncoding: return @"UTF-8";
		case NSUnicodeStringEncoding: return @"UTF-16";
		case NSISO2022JPStringEncoding: return @"JIS";
	}
	return @"no encoding...";
}

- (void)setEncoding:(id)newEncoding
{
	//NSLog(@"encoding!!");
	if([newEncoding isEqualToString:@"S-JIS"]){
		mDocumentEncoding = NSShiftJISStringEncoding;
	}else if([newEncoding isEqualToString:@"EUC"]){
		mDocumentEncoding = NSJapaneseEUCStringEncoding;
	}else if([newEncoding isEqualToString:@"UTF-8"]){
		mDocumentEncoding = NSUTF8StringEncoding;
	}else if([newEncoding isEqualToString:@"UTF-16"]){
		mDocumentEncoding = NSUnicodeStringEncoding;
	}else if([newEncoding isEqualToString:@"JIS"]){
		mDocumentEncoding = NSISO2022JPStringEncoding;
	}else{
		[NSException raise:NSGenericException format:@"MyDocument - setEncoding"];
		return;
	}
	
	[self adjustPopupButtons];
}

- (NSString*)retcode
{
	switch(mReturnCodeType){
		case KLF_type: return @"LF";
		case KCR_type: return @"CR";
		case KCRLF_type: return @"CR-LF";
	}
	[NSException raise:NSGenericException format:@"MyDocument - retcode"];
	return nil;
}

- (void)setRetcode:(NSString*)newRetcode
{
	if([newRetcode isEqualToString:@"LF"]){
		mReturnCodeType = KLF_type;
	}else if([newRetcode isEqualToString:@"CR"]){
		mReturnCodeType = KCR_type;
	}else if([newRetcode isEqualToString:@"CR-LF"]){
		mReturnCodeType = KCRLF_type;
	}else{
		[NSException raise:NSGenericException format:@"MyDocument - setRetcode"];
	}
}

- (NSString*)syntaxtype
{
	switch(mTextType){
		case KTextPlain: return KPlainName;
		case KTextHTML: return KHTMLName;
		case KTextIndex: return KIndexName;
		case KTextKEditMarkup: return KKEditMarkupName;
		case KTextCSS: return KCSSName;
		case KTextPerlScript: return KPerlScriptName;
		case KTextPHP: return KPHPName;
		case KTextRubyScript: return KRubyScriptName;
		
		case KTextShellScript: return KShellScriptName;
	}
	[NSException raise:NSGenericException format:@"MyDocument - syntaxtype"];
	return nil;
}

- (void)setSyntaxtype:(NSString*)newSyntaxType
{
	if([newSyntaxType isEqualToString:KPlainName]){
		[self setTextType:KTextPlain];
	}else if([newSyntaxType isEqualToString:KHTMLName]){
		[self setTextType:KTextHTML];
	}else if([newSyntaxType isEqualToString:KIndexName]){
		[self setTextType:KTextIndex];
	}else if([newSyntaxType isEqualToString:KKEditMarkupName]){
		[self setTextType:KTextKEditMarkup];
	}else if([newSyntaxType isEqualToString:KCSSName]){
		[self setTextType:KTextCSS];
	}else if([newSyntaxType isEqualToString:KPerlScriptName]){
		[self setTextType:KTextPerlScript];
	}else if([newSyntaxType isEqualToString:KRubyScriptName]){
		[self setTextType:KTextRubyScript];
	}else if([newSyntaxType isEqualToString:KPHPName]){
		[self setTextType:KTextPHP];
	
	}else if([newSyntaxType isEqualToString:KShellScriptName]){
		//[self setTextType:KTextShellScript];
	}else{
		[NSException raise:NSGenericException format:@"MyDocument - setSyntaxType"];
		return;
	}
	[self adjustDocumentSettings];
	[self validateColorSyntax];
}

- (NSString*)fontname
{
	return [NSString stringWithString:[self textViewFontName]];
}

- (void)setFontname:(NSString*)newFont
{
	// no error treatment.
	[self setTextViewFontName:newFont];
	[self validateColorSyntax];
}

- (NSNumber*)fontsize
{
	return [NSNumber numberWithFloat:[self textViewFontSize]];
}

- (void)setFontsize:(NSNumber*)newSize
{
	float size = [newSize floatValue];
	if(size >= 1.0){
		[self setTextViewFontSize:size];
		[self validateColorSyntax];
	}else{
		[NSException raise:NSGenericException format:@"MyDocument - setFontsize"];
	}
}

- (NSDictionary*)selectedrange
{
	NSRange range = [myTextView selectedRange];
	return [NSDictionary dictionaryWithObjectsAndKeys:
				[NSNumber numberWithInt:range.location], @"uni_location",
				[NSNumber numberWithInt:range.length], @"uni_length", nil];
}

- (void)setSelectedrange:(NSDictionary*)newSelectedRange
{
	NSRange range;
	unsigned stringLength = [[myTextView string] length];
	NSNumber* location, *length;
	if((location = [newSelectedRange objectForKey:@"uni_location"]) && (length = [newSelectedRange objectForKey:@"uni_length"])){
		range.location = [location intValue];
		range.length = [length intValue];
		if(range.location >= 0 && range.length >= 0 && NSMaxRange(range) <= stringLength){
			[myTextView setSelectedRange:range];
		}else{
			[NSException raise:NSRangeException format:@"Given Range is not Valid."];
		}
			
	}
}

- (NSNumber*)wordwrap
{
	return [NSNumber numberWithBool:[self isWordWrapped]];
}

- (void)setWordwrap:(NSNumber*)newWordWrap
{
	[self setWordWrapped:[newWordWrap boolValue]];
}

- (NSNumber*)autoindent
{
	return [NSNumber numberWithBool:[self isAutoIndented]];
}

- (void)setAutoindent:(NSNumber*)newAutoIndent
{
	[self setAutoIndented:[newAutoIndent boolValue]];
}

- (NSTextStorage*)selectedtext
{
	//NSString* selectedText = [[myTextView string] substringWithRange:[myTextView selectedRange]];
	//return [NSString stringWithString:selectedText];
	NSAttributedString* attrString = [[myTextView textStorage] attributedSubstringFromRange:[myTextView selectedRange]];
	NSTextStorage * storage = [[NSTextStorage alloc] initWithAttributedString:attrString];
	return [storage autorelease];
}

- (void)setSelectedtext:(NSTextStorage*)newText
{
	if(newText){
		NSRange selectedRange = [myTextView selectedRange];
		[myTextView replaceCharactersInRange:selectedRange withString:[newText string]];
		[myTextView setSelectedRange:NSMakeRange(selectedRange.location, [newText length])];
	}
}

- (NSNumber*)selectionLocation
{
	return [NSNumber numberWithInt:[myTextView selectedRange].location];
}

- (void)setSelectionLocation:(NSNumber*)newSelectionLocation
{
	NSRange selectedRange = [myTextView selectedRange];
	int maxLength = [[myTextView string] length];
	int newLocation = [newSelectionLocation intValue];
	if(newLocation > maxLength){
		[NSException raise:NSGenericException format:@"MyDocument - setSelectionLocation"];
		return;
	}
	if(newLocation + selectedRange.length > maxLength){
		selectedRange.length = maxLength - newLocation;
	}
	selectedRange.location = newLocation;
	[myTextView setSelectedRange:selectedRange];
}

- (NSNumber*)selectionLength
{
	return [NSNumber numberWithInt:[myTextView selectedRange].length];
}

- (void)setSelectionLength:(NSNumber*)newSelectionLength
{
	NSRange selectedRange = [myTextView selectedRange];
	selectedRange.length = [newSelectionLength intValue];
	int maxLength = [[myTextView string] length];

	if(selectedRange.length > maxLength - selectedRange.location){
		[NSException raise:NSGenericException format:@"MyDocument - setSelectionLength"];
		return;
	}
	[myTextView setSelectedRange:selectedRange];
}

- (NSTextStorage*)text
{
	return [myTextView textStorage];
}




- (NSNumber*)handleReplaceAllCommand:(NSScriptCommand*)command
{
	NSDictionary* dict = [command evaluatedArguments];
	
	BOOL ignoreCase = YES;
	NSString* searchText = nil;
	NSString* replaceText = nil;
	
	if(!(searchText = [dict objectForKey:@"searchText"]) 
			|| !(replaceText = [dict objectForKey:@"replaceText"])
			|| [searchText isEqualToString:@""]){
		[NSException raise:NSGenericException format:@"MyDocument - handleReplaceAllCommand:"];
	}
	
	id curObj;
	if(curObj = [dict objectForKey:@"ignoreCase"]){
		ignoreCase = [curObj boolValue];
	}
	KSearchServer* searchServer = [[KSearchServer alloc] init];
	[searchServer setSearchString:searchText];
	[searchServer setReplaceString:replaceText];
	[searchServer setIgnoreCase:ignoreCase];
	
	unsigned long replaceCount = [searchServer replaceAll:myTextView];
	
	[searchServer release];
	
	return [NSNumber numberWithLong:replaceCount];
}

- (NSNumber*)handleSearchCommand:(NSScriptCommand*)command
{
	//NSLog(@"search!! = %@", [command description]);
	NSDictionary* dict = [command evaluatedArguments];
	
	BOOL ignoreCase = YES;
	NSString* searchText = nil;
	unsigned long fromOffset = 0;
	id curObj;
	if(!(searchText = [dict objectForKey:@"searchText"])
			|| [searchText isEqualToString:@""]){
		[NSException raise:NSGenericException format:@"MyDocument - handleSearchCommand:"];
	}
	if(curObj = [dict objectForKey:@"ignoreCase"]){
		ignoreCase = [curObj boolValue];
	}
	if(curObj = [dict objectForKey:@"fromOffset"]){
		fromOffset = [curObj longValue];
		if(fromOffset > [[myTextView string] length]){
			[NSException raise:NSGenericException format:@"MyDocument - illegal 'fromOffset'"];
		}
	}
	
	KSearchServer* searchServer = [[KSearchServer alloc] init];
	[searchServer setSearchString:searchText];
	[searchServer setIgnoreCase:ignoreCase];
	
	NSRange currentSelectedRange = [myTextView selectedRange];
	[myTextView setSelectedRange:NSMakeRange(fromOffset, 0)];
	NSRange newSelection = [searchServer searchString:myTextView direction:KSearchForward];
	[myTextView setSelectedRange:currentSelectedRange];
	
	[searchServer release];
	
	if(newSelection.location == NSNotFound){
		return [NSNumber numberWithLong:-1];
	}
	
	return [NSNumber numberWithLong:newSelection.location];
}

- (void)handleScrollToSelectionCommand:(NSScriptCommand*)command
{
	[self scrollToSelection:nil];
}


@end

















